@@ -22,8 +22,7 @@
 #define MAX_ALLOCATED_NDIS_PACKETS 20
 #define MAX_ALLOCATED_NDIS_BUFFERS 20
 
-extern struct nt_list wrap_drivers;
-extern NT_SPIN_LOCK ntoskernel_lock, loader_lock;
+extern NT_SPIN_LOCK loader_lock;
 
 static struct workqueue_struct *ndis_wq;
 static void ndis_worker(void *dummy);
@@ -31,8 +30,6 @@ static struct work_struct ndis_work;
 static struct nt_list ndis_worker_list;
 static NT_SPIN_LOCK ndis_work_list_lock;
 
-static void dump_ndis_buffer(ndis_buffer *buf);
-
 /* ndis_init is called once when module is loaded */
 int ndis_init(void)
 {
@@ -121,10 +118,9 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterMiniport)
 		TRACEEXIT1(return NDIS_STATUS_BAD_CHARACTERISTICS);
 	}
 
-	DBGTRACE1("Version %d.%d", miniport_char->major_version,
-		  miniport_char->minor_version);
-	DBGTRACE1("Len: %08x:%u", char_len, (u32)sizeof(struct miniport_char));
-
+	DBGTRACE1("%d.%d, %d, %u", miniport_char->major_version,
+		  miniport_char->minor_version, char_len,
+		  (u32)sizeof(struct miniport_char));
 	wrap_driver =
 		IoGetDriverObjectExtension(drv_obj,
 					   (void *)CE_WRAP_DRIVER_CLIENT_ID);
@@ -139,7 +135,7 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterMiniport)
 	    STATUS_SUCCESS)
 		TRACEEXIT1(return NDIS_STATUS_RESOURCES);
 	wrap_driver->ndis_driver = ndis_driver;
-	TRACEENTER1("ndis_driver: %p", ndis_driver);
+	TRACEENTER1("driver: %p", ndis_driver);
 	memcpy(&ndis_driver->miniport, miniport_char,
 	       char_len > sizeof(*miniport_char) ?
 	       sizeof(*miniport_char) : char_len);
@@ -158,7 +154,7 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterMiniport)
 		func = (void **)&ndis_driver->miniport.query;
 		for (i = 0; i < (sizeof(miniport_funcs) /
 				 sizeof(miniport_funcs[0])); i++)
-			DBGTRACE2("miniport function '%s' is at %p",
+			DBGTRACE2("function '%s' is at %p",
 				  miniport_funcs[i], func[i]);
 	}
 	TRACEEXIT1(return NDIS_STATUS_SUCCESS);
@@ -188,7 +184,7 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterDevice)
 
 	*dev_obj = tmp;
 	*dev_obj_handle = *dev_obj;
-	for (i = 0; i <= IRP_MJ_MAXIMUM_FUNCTION; i++)
+	for (i = 0; i < IRP_MJ_MAXIMUM_FUNCTION; i++)
 		if (funcs[i] && i != IRP_MJ_PNP && i != IRP_MJ_POWER) {
 			drv_obj->major_func[i] = funcs[i];
 			DBGTRACE1("mj_fn for 0x%x is at %p", i, funcs[i]);
@@ -220,7 +216,6 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisAllocateMemory)
 	(void **dest, UINT length, UINT flags,
 	 NDIS_PHY_ADDRESS highest_address)
 {
-	DBGTRACE4("length = %u, flags = %08X", length, flags);
 	return NdisAllocateMemoryWithTag(dest, length, 0);
 }
 
@@ -258,7 +253,7 @@ wstdcall void WRAP_EXPORT(NdisOpenConfiguration)
 	(NDIS_STATUS *status, struct ndis_miniport_block **conf_handle,
 	 struct ndis_miniport_block *handle)
 {
-	TRACEENTER2("confHandle: %p", conf_handle);
+	TRACEENTER2("%p", conf_handle);
 	*conf_handle = handle;
 	*status = NDIS_STATUS_SUCCESS;
 	TRACEEXIT2(return);
@@ -268,7 +263,7 @@ wstdcall void WRAP_EXPORT(NdisOpenProtocolConfiguration)
 	(NDIS_STATUS *status, void **confhandle,
 	 struct unicode_string *section)
 {
-	TRACEENTER2("confHandle: %p", confhandle);
+	TRACEENTER2("%p", confhandle);
 	*status = NDIS_STATUS_SUCCESS;
 	TRACEEXIT2(return);
 }
@@ -280,11 +275,9 @@ wstdcall void WRAP_EXPORT(NdisOpenConfigurationKeyByName)
 	struct ansi_string ansi;
 	TRACEENTER2("");
 	if (RtlUnicodeStringToAnsiString(&ansi, key, TRUE) == STATUS_SUCCESS) {
-		DBGTRACE2("key: %s", ansi.buf);
+		DBGTRACE2("%s", ansi.buf);
 		RtlFreeAnsiString(&ansi);
-	} else
-		DBGTRACE2("couldn't convert ustring %d, %d, %p",
-			  key->length, key->max_length, key->buf);
+	}
 	*subkeyhandle = handle;
 	*status = NDIS_STATUS_SUCCESS;
 	TRACEEXIT2(return);
@@ -294,7 +287,7 @@ wstdcall void WRAP_EXPORT(NdisOpenConfigurationKeyByIndex)
 	(NDIS_STATUS *status, void *handle, ULONG index,
 	 struct unicode_string *key, void **subkeyhandle)
 {
-	TRACEENTER2("index: %u", index);
+	TRACEENTER2("%u", index);
 //	*subkeyhandle = handle;
 	*status = NDIS_STATUS_FAILURE;
 	TRACEEXIT2(return);
@@ -306,7 +299,7 @@ wstdcall void WRAP_EXPORT(NdisCloseConfiguration)
 	/* instead of freeing all configuration parameters as we are
 	 * supposed to do here, we free them when the device is
 	 * removed */
-	TRACEENTER2("handle: %p", handle);
+	TRACEENTER2("%p", handle);
 	return;
 }
 
@@ -318,18 +311,14 @@ wstdcall void WRAP_EXPORT(NdisOpenFile)
 	struct ansi_string ansi;
 	struct wrap_bin_file *bin_file;
 
-	TRACEENTER2("status = %p, filelength = %p, *filelength = %d, "
-		    "high = %llx, file = %p, *file = %p",
-		    status, filelength, *filelength,
-		    highest_address, file, *file);
-
+	TRACEENTER2("%p, %d, %llx, %p", status, *filelength,
+		    highest_address, *file);
 	if (RtlUnicodeStringToAnsiString(&ansi, filename, TRUE) !=
 	    STATUS_SUCCESS) {
 		*status = NDIS_STATUS_RESOURCES;
 		TRACEEXIT2(return);
 	}
-	DBGTRACE2("Filename: %s", ansi.buf);
-
+	DBGTRACE2("%s", ansi.buf);
 	bin_file = get_bin_file(ansi.buf);
 	if (bin_file) {
 		*file = bin_file;
@@ -345,7 +334,7 @@ wstdcall void WRAP_EXPORT(NdisOpenFile)
 wstdcall void WRAP_EXPORT(NdisMapFile)
 	(NDIS_STATUS *status, void **mappedbuffer, struct wrap_bin_file *file)
 {
-	TRACEENTER2("handle: %p", file);
+	TRACEENTER2("%p", file);
 
 	if (!file) {
 		*status = NDIS_STATUS_ALREADY_MAPPED;
@@ -360,14 +349,14 @@ wstdcall void WRAP_EXPORT(NdisMapFile)
 wstdcall void WRAP_EXPORT(NdisUnmapFile)
 	(struct wrap_bin_file *file)
 {
-	TRACEENTER2("handle: %p", file);
+	TRACEENTER2("%p", file);
 	TRACEEXIT2(return);
 }
 
 wstdcall void WRAP_EXPORT(NdisCloseFile)
 	(struct wrap_bin_file *file)
 {
-	TRACEENTER2("handle: %p", file);
+	TRACEENTER2("%p", file);
 	free_bin_file(file);
 	TRACEEXIT2(return);
 }
@@ -399,7 +388,7 @@ wstdcall ULONG WRAP_EXPORT(NDIS_BUFFER_TO_SPAN_PAGES)
 wstdcall void WRAP_EXPORT(NdisGetBufferPhysicalArraySize)
 	(ndis_buffer *buffer, UINT *arraysize)
 {
-	TRACEENTER3("Buffer: %p", buffer);
+	TRACEENTER3("%p", buffer);
 	*arraysize = NDIS_BUFFER_TO_SPAN_PAGES(buffer);
 	TRACEEXIT3(return);
 }
@@ -424,19 +413,19 @@ ndis_encode_setting(struct wrap_device_setting *setting,
 		ERROR("couldn't allocate memory");
 		return NULL;
 	}
-	TRACEENTER2("type = %d", type);
+	TRACEENTER2("%d", type);
 	switch(type) {
 	case NdisParameterInteger:
 		param->data.integer = simple_strtol(setting->value, NULL, 0);
-		DBGTRACE2("value = %u", (ULONG)param->data.integer);
+		DBGTRACE2("%u", (ULONG)param->data.integer);
 		break;
 	case NdisParameterHexInteger:
 		param->data.integer = simple_strtol(setting->value, NULL, 16);
-		DBGTRACE2("value = %u", (ULONG)param->data.integer);
+		DBGTRACE2("%u", (ULONG)param->data.integer);
 		break;
 	case NdisParameterString:
 		RtlInitAnsiString(&ansi, setting->value);
-		DBGTRACE2("setting value = '%s'", ansi.buf);
+		DBGTRACE2("'%s'", ansi.buf);
 		if (RtlAnsiStringToUnicodeString(&param->data.string,
 						 &ansi, TRUE)) {
 			ExFreePool(param);
@@ -500,21 +489,19 @@ wstdcall void WRAP_EXPORT(NdisReadConfiguration)
 
 	TRACEENTER2("nmb: %p", nmb);
 	ret = RtlUnicodeStringToAnsiString(&ansi, key, TRUE);
-	DBGTRACE3("rtl func returns: %d", ret);
 	if (ret || ansi.buf == NULL) {
 		*param = NULL;
 		*status = NDIS_STATUS_FAILURE;
 		RtlFreeAnsiString(&ansi);
 		TRACEEXIT2(return);
 	}
-	DBGTRACE3("wd: %p, string: %s", nmb->wnd->wd, ansi.buf);
+	DBGTRACE3("%s", ansi.buf);
 	keyname = ansi.buf;
 
 	nt_spin_lock(&loader_lock);
 	nt_list_for_each_entry(setting, &nmb->wnd->wd->settings, list) {
 		if (strnicmp(keyname, setting->name, ansi.length) == 0) {
-			DBGTRACE2("setting found %s=%s",
-				  keyname, setting->value);
+			DBGTRACE2("setting %s=%s", keyname, setting->value);
 			nt_spin_unlock(&loader_lock);
 			*param = ndis_encode_setting(setting, type);
 			if (*param)
@@ -522,7 +509,7 @@ wstdcall void WRAP_EXPORT(NdisReadConfiguration)
 			else
 				*status = NDIS_STATUS_FAILURE;
 			RtlFreeAnsiString(&ansi);
-			DBGTRACE2("status = %d", *status);
+			DBGTRACE2("%d", *status);
 			TRACEEXIT2(return);
 		}
 	}
@@ -548,7 +535,7 @@ wstdcall void WRAP_EXPORT(NdisWriteConfiguration)
 		TRACEEXIT2(return);
 	}
 	keyname = ansi.buf;
-	DBGTRACE2("key = %s", keyname);
+	DBGTRACE2("%s", keyname);
 
 	nt_spin_lock(&loader_lock);
 	nt_list_for_each_entry(setting, &nmb->wnd->wd->settings, list) {
@@ -592,9 +579,13 @@ wstdcall void WRAP_EXPORT(NdisInitializeString)
 	struct ansi_string ansi;
 
 	TRACEENTER2("");
-	RtlInitAnsiString(&ansi, src);
-	if (RtlAnsiStringToUnicodeString(dest, &ansi, TRUE))
-		DBGTRACE2("failed");
+	if (src == NULL) {
+		dest->length = dest->max_length = 0;
+		dest->buf = NULL;
+	} else {
+		RtlInitAnsiString(&ansi, src);
+		RtlAnsiStringToUnicodeString(dest, &ansi, TRUE);
+	}
 	TRACEEXIT2(return);
 }
 
@@ -605,45 +596,35 @@ wstdcall void WRAP_EXPORT(NdisInitAnsiString)
 	TRACEEXIT2(return);
 }
 
-wstdcall void WRAP_EXPORT(NdisInitString)
-	(struct ansi_string *dst, CHAR *src)
-{
-	RtlInitString(dst, src);
-	TRACEEXIT2(return);
-}
-
 wstdcall void WRAP_EXPORT(NdisInitUnicodeString)
 	(struct unicode_string *dest, const wchar_t *src)
 {
-	TRACEEXIT2(return RtlInitUnicodeString(dest, src));
+	RtlInitUnicodeString(dest, src);
+	return;
 }
 
 wstdcall NDIS_STATUS WRAP_EXPORT(NdisAnsiStringToUnicodeString)
 	(struct unicode_string *dst, struct ansi_string *src)
 {
-	BOOLEAN alloc;
 	TRACEENTER2("");
 	if (dst == NULL || src == NULL)
 		TRACEEXIT2(return NDIS_STATUS_FAILURE);
-	if (dst->buf == NULL)
-		alloc = TRUE;
+	if (RtlAnsiStringToUnicodeString(dst, src, FALSE) == STATUS_SUCCESS)
+		return NDIS_STATUS_SUCCESS;
 	else
-		alloc = FALSE;
-	TRACEEXIT2(return RtlAnsiStringToUnicodeString(dst, src, alloc));
+		return NDIS_STATUS_FAILURE;
 }
 
 wstdcall NDIS_STATUS WRAP_EXPORT(NdisUnicodeStringToAnsiString)
 	(struct ansi_string *dst, struct unicode_string *src)
 {
-	BOOLEAN alloc;
 	TRACEENTER2("");
 	if (dst == NULL || src == NULL)
 		TRACEEXIT2(return NDIS_STATUS_FAILURE);
-	if (dst->buf == NULL)
-		alloc = TRUE;
+	if (RtlUnicodeStringToAnsiString(dst, src, FALSE) == STATUS_SUCCESS)
+		return NDIS_STATUS_SUCCESS;
 	else
-		alloc = FALSE;
-	TRACEEXIT2(return RtlUnicodeStringToAnsiString(dst, src, alloc));
+		return NDIS_STATUS_FAILURE;
 }
 
 wstdcall void WRAP_EXPORT(NdisMSetAttributesEx)
@@ -741,7 +722,7 @@ wstdcall void WRAP_EXPORT(NdisMQueryAdapterResources)
 	resource_length = sizeof(struct cm_partial_resource_list) +
 		sizeof(struct cm_partial_resource_descriptor) *
 		(list->count - 1);
-	DBGTRACE2("wnd: %p. buf: %p, len: %d (%d), %p %d %d", wnd,
+	DBGTRACE2("%p, %p,%d (%d), %p %d %d", wnd,
 		  resource_list, *size, resource_length,
 		  &list->partial_descriptors[list->count-1],
 		  list->partial_descriptors[list->count-1].u.interrupt.level,
@@ -784,8 +765,7 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMMapIoSpace)
 
 	wnd->mem_start = phy_addr;
 	wnd->mem_end = phy_addr + len;
-
-	DBGTRACE2("ioremap successful %p", *virt);
+	DBGTRACE2("%p", *virt);
 	TRACEEXIT2(return NDIS_STATUS_SUCCESS);
 }
 
@@ -907,7 +887,7 @@ wstdcall void WRAP_EXPORT(NdisMAllocateSharedMemory)
 	dma_addr_t dma_addr;
 	struct wrap_device *wd = nmb->wnd->wd;
 
-	TRACEENTER3("map count: %d, size: %u, cached: %d",
+	TRACEENTER3("count: %d, size: %u, cached: %d",
 		    nmb->wnd->dma_map_count, size, cached);
 
 	*virt = PCI_DMA_ALLOC_COHERENT(wd->pci.pdev, size, &dma_addr);
@@ -935,7 +915,7 @@ static void ndis_worker(void *dummy)
 		ndis_work_entry = container_of(ent, struct ndis_work_entry,
 					       list);
 		ndis_work_item = ndis_work_entry->ndis_work_item;
-		WORKTRACE("%p: calling work at %p with %p", ndis_work_item,
+		WORKTRACE("%p: %p, %p", ndis_work_item,
 			  ndis_work_item->func, ndis_work_item->ctx);
 		LIN2WIN2(ndis_work_item->func, ndis_work_item,
 			 ndis_work_item->ctx);
@@ -1310,7 +1290,10 @@ wstdcall void WRAP_EXPORT(NdisMStartBufferPhysicalMapping)
 
 	DBGTRACE3("%p, %p, %u", buf, MmGetSystemAddressForMdl(buf),
 		  MmGetMdlByteCount(buf));
-	dump_ndis_buffer(buf);
+	DBG_BLOCK(4) {
+		dump_bytes(__FUNCTION__, MmGetSystemAddressForMdl(buf),
+			   MmGetMdlByteCount(buf));
+	}
 	// map buffer
 	phy_addr_array[0].phy_addr =
 		PCI_DMA_MAP_SINGLE(wnd->wd->pci.pdev,
@@ -1621,12 +1604,9 @@ wstdcall void WRAP_EXPORT(NdisSend)
 
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	if (miniport->send_packets) {
-		struct ndis_packet *packets[1];
-
-		packets[0] = packet;
 		irql = raise_irql(DISPATCH_LEVEL);
 		LIN2WIN3(miniport->send_packets, wnd->nmb->adapter_ctx,
-			 packets, 1);
+			 &packet, 1);
 		lower_irql(irql);
 		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE)) {
 			struct ndis_packet_oob_data *oob_data;
@@ -1795,31 +1775,27 @@ wstdcall void WRAP_EXPORT(NdisMDeregisterAdapterShutdownHandler)
 
 static void ndis_irq_handler(unsigned long data)
 {
-	struct ndis_irq *ndis_irq = (struct ndis_irq *)data;
-	struct wrap_ndis_device *wnd;
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	struct miniport_char *miniport;
 
-	wnd = ndis_irq->wnd;
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	LIN2WIN1(miniport->handle_interrupt, wnd->nmb->adapter_ctx);
 	if (miniport->enable_interrupts)
 		LIN2WIN1(miniport->enable_interrupts, wnd->nmb->adapter_ctx);
 }
 
-static irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs)
+irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs)
 {
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	int recognized, queue_handler;
-	struct ndis_irq *ndis_irq = (struct ndis_irq *)data;
-	struct wrap_ndis_device *wnd;
 	struct miniport_char *miniport;
 
-	wnd = ndis_irq->wnd;
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	/* this spinlock should be shared with NdisMSynchronizeWithInterrupt
 	 */
-	nt_spin_lock(&ndis_irq->lock);
+	nt_spin_lock(&wnd->ndis_irq->lock);
 	recognized = queue_handler = 0;
-	if (ndis_irq->req_isr)
+	if (wnd->ndis_irq->req_isr)
 		LIN2WIN3(miniport->isr, &recognized, &queue_handler,
 			 wnd->nmb->adapter_ctx);
 	else { //if (miniport->disable_interrupts)
@@ -1827,7 +1803,7 @@ static irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs)
 		/* it is not shared interrupt, so handler must be called */
 		recognized = queue_handler = 1;
 	}
-	nt_spin_unlock(&ndis_irq->lock);
+	nt_spin_unlock(&wnd->ndis_irq->lock);
 
 	if (recognized) {
 		if (queue_handler)
@@ -1855,10 +1831,9 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterInterrupt)
 	nt_spin_lock_init(&ndis_irq->lock);
 	wnd->ndis_irq = ndis_irq;
 
-	tasklet_init(&wnd->irq_tasklet, ndis_irq_handler,
-		     (unsigned long)ndis_irq);
+	tasklet_init(&wnd->irq_tasklet, ndis_irq_handler, (unsigned long)wnd);
 	if (request_irq(vector, ndis_isr, req_isr ? SA_SHIRQ : 0,
-			"ndiswrapper", ndis_irq)) {
+			"ndiswrapper", wnd)) {
 		printk(KERN_WARNING "%s: request for irq %d failed\n",
 		       DRIVER_NAME, vector);
 		TRACEEXIT1(return NDIS_STATUS_RESOURCES);
@@ -1881,7 +1856,7 @@ wstdcall void WRAP_EXPORT(NdisMDeregisterInterrupt)
 	if (!wnd)
 		TRACEEXIT1(return);
 
-	free_irq(ndis_irq->irq.irq, ndis_irq);
+	free_irq(ndis_irq->irq.irq, wnd);
 	tasklet_kill(&wnd->irq_tasklet);
 	ndis_irq->enabled = 0;
 	ndis_irq->wnd = NULL;
@@ -1989,7 +1964,7 @@ wstdcall void WRAP_EXPORT(NdisMIndicateStatus)
 					&cand->candidates[i];
 				if ((u8 *)(c + 1) > end) {
 					DBGTRACE2("Truncated "
-						    "PMKID_CANDIDATE_LIST");
+						  "PMKID_CANDIDATE_LIST");
 					break;
 				}
 				DBGTRACE2("%ld: " MACSTR " 0x%lx",
@@ -2465,23 +2440,20 @@ wstdcall struct nt_list *WRAP_EXPORT(NdisInterlockedInsertHeadList)
 	(struct nt_list *head, struct nt_list *entry,
 	 struct ndis_spinlock *lock)
 {
-	return ExInterlockedInsertHeadList(head, entry,
-				&lock->klock);
+	return ExInterlockedInsertHeadList(head, entry, &lock->klock);
 }
 
 wstdcall struct nt_list *WRAP_EXPORT(NdisInterlockedInsertTailList)
 	(struct nt_list *head, struct nt_list *entry,
 	 struct ndis_spinlock *lock)
 {
-	return ExInterlockedInsertTailList(head, entry,
-				&lock->klock);
+	return ExInterlockedInsertTailList(head, entry, &lock->klock);
 }
 
 wstdcall struct nt_list *WRAP_EXPORT(NdisInterlockedRemoveHeadList)
 	(struct nt_list *head, struct ndis_spinlock *lock)
 {
-	return ExInterlockedRemoveHeadList(head,
-				&lock->klock);
+	return ExInterlockedRemoveHeadList(head, &lock->klock);
 }
 
 wstdcall NDIS_STATUS WRAP_EXPORT(NdisMInitializeScatterGatherDma)
@@ -2558,8 +2530,7 @@ NdisMResetComplete(struct ndis_miniport_block *nmb, NDIS_STATUS status,
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
 
-	TRACEENTER3("status: %08X, reset status: %u", status,
-		    address_reset);
+	TRACEENTER3("status: %08X, %u", status, address_reset);
 	wnd->ndis_comm_status = status;
 	wnd->ndis_comm_done = 1 + address_reset;
 	wake_up(&wnd->ndis_comm_wq);
@@ -2644,25 +2615,11 @@ wstdcall ULONG WRAP_EXPORT(NdisWritePcmciaAttributeMemory)
 
  /* Unimplemented...*/
 wstdcall void WRAP_EXPORT(NdisMSetAttributes)(void){UNIMPL();}
-wstdcall void WRAP_EXPORT(EthFilterDprIndicateReceiveComplete)
-	(void){UNIMPL();}
+wstdcall void WRAP_EXPORT(EthFilterDprIndicateReceiveComplete)(void){UNIMPL();}
 wstdcall void WRAP_EXPORT(EthFilterDprIndicateReceive)(void){UNIMPL();}
-wstdcall void WRAP_EXPORT(NdisMRemoveMiniport)(void) { UNIMPL(); }
+wstdcall void WRAP_EXPORT(NdisMRemoveMiniport)(void){UNIMPL();}
 wstdcall void WRAP_EXPORT(NdisMCoActivateVcComplete)(void){UNIMPL();}
-
-wstdcall void WRAP_EXPORT(NdisMCoDeactivateVcComplete)(void)
-{
-	UNIMPL();
-	return;
-}
-
-static void dump_ndis_buffer(ndis_buffer *buf)
-{
-	DBG_BLOCK(4) {
-		dump_bytes(__FUNCTION__, MmGetSystemAddressForMdl(buf),
-			   MmGetMdlByteCount(buf));
-	}
-}
+wstdcall void WRAP_EXPORT(NdisMCoDeactivateVcComplete)(void){UNIMPL();}
 
 #include "ndis_exports.h"
 
@@ -2679,20 +2636,5 @@ void init_nmb_functions(struct ndis_miniport_block *nmb)
 	nmb->eth_rx_indicate = WRAP_FUNC_PTR(EthRxIndicateHandler);
 	nmb->eth_rx_complete = WRAP_FUNC_PTR(EthRxComplete);
 	nmb->td_complete = WRAP_FUNC_PTR(NdisMTransferDataComplete);
-
-	/* TODO: setup pointers to
-
-	   NdisMWanSendComplete
-	   NdisMWanIndicateReceive
-	   NdisMWanIndicateReceiveComplete
-	   
-	   NdisMTrIndicateReceive
-	   NdisMTrIndicateReceiveComplete
-
-	   NdisMFddiIndicateReceive
-	   NdisMFddiIndicateReceiveComplete
-	   NdisMArcIndicateReceive
-	   NdisMArcIndicateReceiveComplete
-	 */
 }
 
