@@ -680,7 +680,11 @@ struct auth_encr_capa {
 	unsigned long encr;
 };
 
-enum driver_type { DRIVER_WIRELESS, DRIVER_ETHERNET, };
+enum driver_type { DRIVER_WIRELESS = 1, DRIVER_ETHERNET, };
+
+enum hw_status {
+	HW_INITIALIZED = 1, HW_SUSPENDED, HW_HALTED,
+};
 
 /*
  * This struct contains function pointers that the drivers references
@@ -849,6 +853,7 @@ struct wrap_ndis_device {
 	struct ndis_tcp_ip_checksum_packet_info tx_csum_info;
 	enum ndis_physical_medium physical_medium;
 	u32 ndis_wolopts;
+	BOOLEAN pm_capa;
 };
 
 struct ndis_pmkid_candidate {
@@ -896,6 +901,7 @@ wstdcall void NdisMResetComplete(struct ndis_miniport_block *nmb,
 wstdcall ULONG NDIS_BUFFER_TO_SPAN_PAGES(ndis_buffer *buffer);
 wstdcall BOOLEAN NdisWaitEvent(struct ndis_event *event, UINT timeout);
 wstdcall void NdisSetEvent(struct ndis_event *event);
+irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs);
 wstdcall void NdisMDeregisterInterrupt(struct ndis_irq *ndis_irq);
 wstdcall void EthRxIndicateHandler
 	(struct ndis_miniport_block *nmb, void *rx_ctx, char *header1,
