@@ -142,8 +142,7 @@ void cancel_delayed_work(struct work_struct *work_struct);
 #ifndef in_atomic
 #ifdef CONFIG_PREEMPT
 #define in_atomic()							\
-	(in_interrupt() ||						\
-	 (preempt_get_count() & ~PREEMPT_ACTIVE) != kernel_locked())
+	 ((preempt_get_count() & ~PREEMPT_ACTIVE) != 0)
 #else
 #define in_atomic() (in_interrupt())
 #endif // CONFIG_PREEMPT
@@ -507,9 +506,6 @@ struct pe_image {
 	IMAGE_OPTIONAL_HEADER *opt_hdr;
 };
 
-extern NT_SPIN_LOCK atomic_lock;
-extern NT_SPIN_LOCK cancel_lock;
-
 struct wrap_timer {
 	long repeat;
 	struct nt_list list;
@@ -559,11 +555,6 @@ struct wrap_driver {
 	struct wrap_ndis_driver *ndis_driver;
 };
 
-enum hw_status {
-	HW_NORMAL, HW_SUSPENDED, HW_HALTED, HW_RMMOD, HW_AVAILABLE,
-	HW_INITIALIZED,
-};
-
 struct wrap_device {
 	/* first part is (de)initialized once by loader */
 	int dev_bus_type;
@@ -662,7 +653,7 @@ wstdcall UCHAR READ_PORT_UCHAR(ULONG_PTR port);
 #undef ExAllocatePoolWithTag
 wstdcall void *ExAllocatePoolWithTag(enum pool_type pool_type, SIZE_T size,
 				    ULONG tag);
-#ifdef ALLOC_DEBUG
+#if defined(ALLOC_DEBUG) && ALLOC_DEBUG > 1
 #define ExAllocatePoolWithTag(pool_type, size, tag)			\
 	wrap_ExAllocatePoolWithTag(pool_type, size, tag, __FILE__, __LINE__)
 #endif
