@@ -19,7 +19,6 @@
 #include "loader.h"
 
 extern NT_SPIN_LOCK loader_lock;
-extern struct nt_list ndis_drivers;
 extern struct wrap_device *wrap_devices;
 
 static NTSTATUS start_pdo(struct device_object *pdo)
@@ -403,12 +402,15 @@ NTSTATUS pnp_set_device_power_state(struct wrap_device *wd,
 	if (state > PowerDeviceD0) {
 		status = IoSendIrpTopDev(pdo, IRP_MJ_POWER, IRP_MN_QUERY_POWER,
 					 &irp_sl);
-		if (status != STATUS_SUCCESS)
-			DBGTRACE1("query power returns %08X", status);
+		if (status != STATUS_SUCCESS) {
+			DBGTRACE1("query of power to %d returns %08X",
+				  state, status);
+			TRACEEXIT1(return status);
+		}
 	}
 	status = IoSendIrpTopDev(pdo, IRP_MJ_POWER, IRP_MN_SET_POWER, &irp_sl);
 	if (status != STATUS_SUCCESS)
-		WARNING("set power returns %08X", status);
+		WARNING("setting power to %d failed: %08X", state, status);
 	TRACEEXIT1(return status);
 }
 
@@ -670,7 +672,7 @@ void *wrap_pnp_start_usb_device(struct usb_device *udev,
 	if (wd->usb.intf) {
 		DBGTRACE1("device already initialized: %p", wd->usb.intf);
 		usb_set_intfdata(intf, NULL);
-		ret = -EINVAL;
+		ret = 0;
 	} else {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 		wd->usb.udev = interface_to_usbdev(intf);
