@@ -69,11 +69,25 @@ static unsigned int urb_id = 0;
 #define USB_CTRL_GET_TIMEOUT 5000
 #endif
 
+#ifndef URB_NO_TRANSFER_DMA_MAP
+#define URB_NO_TRANSFER_DMA_MAP 0
+#endif
+
+/* wrap_urb->flags */
+/* transfer_buffer for urb is allocated; free it in wrap_free_urb */
+#define WRAP_URB_COPY_BUFFER 0x01
+
 static int inline wrap_cancel_urb(struct wrap_urb *wrap_urb)
 {
 	int ret;
+	USBTRACE("%p, %p, %d", wrap_urb, wrap_urb->urb, wrap_urb->state);
+	if (wrap_urb->state != URB_SUBMITTED)
+		USBEXIT(return -1);
 	ret = usb_unlink_urb(wrap_urb->urb);
 	if (ret == -EINPROGRESS) {
+		if (wrap_urb->state != URB_SUBMITTED)
+			WARNING("urb %p in wrong state: %d",
+				wrap_urb->urb, wrap_urb->state);
 		wrap_urb->state = URB_CANCELED;
 		return 0;
 	} else {
@@ -84,12 +98,12 @@ static int inline wrap_cancel_urb(struct wrap_urb *wrap_urb)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 
-#define URB_NO_TRANSFER_DMA_MAP		0x0004
 #define URB_STATUS(wrap_urb) (wrap_urb->urb_status)
 
 static void *usb_buffer_alloc(struct usb_device *udev, size_t size,
 			      unsigned mem_flags, dma_addr_t *dma)
 {
+	*dma = 0;
 	return kmalloc(size, mem_flags);
 }
 
@@ -104,6 +118,11 @@ static void usb_init_urb(struct urb *urb)
 	memset(urb, 0, sizeof(*urb));
 }
 
+static void usb_kill_urb(struct urb *urb)
+{
+	usb_unlink_urb(urb);
+}
+
 #else
 
 #define URB_STATUS(wrap_urb) (wrap_urb->urb->status)
@@ -112,7 +131,7 @@ static void usb_init_urb(struct urb *urb)
 
 static struct nt_list wrap_urb_complete_list;
 static NT_SPIN_LOCK wrap_urb_complete_list_lock;
-static STDCALL void wrap_cancel_irp(struct device_object *dev_obj,
+static wstdcall void wrap_cancel_irp(struct device_object *dev_obj,
 				    struct irp *irp);
 
 /* use tasklet instead worker to process completed urbs */
@@ -148,11 +167,9 @@ void usb_exit(void)
 #ifdef USB_TASKLET
 	tasklet_kill(&wrap_urb_complete_work);
 #else
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 	cancel_delayed_work(&wrap_urb_complete_work);
 #endif
-#endif
-	TRACEEXIT1(return);
+	USBEXIT(return);
 }
 
 int usb_init_device(struct wrap_device *wd)
@@ -162,7 +179,7 @@ int usb_init_device(struct wrap_device *wd)
 	USBEXIT(return 0);
 }
 
-void usb_exit_device(struct wrap_device *wd)
+static void kill_all_urbs(struct wrap_device *wd, int complete)
 {
 	struct nt_list *ent;
 	struct wrap_urb *wrap_urb;
@@ -179,12 +196,19 @@ void usb_exit_device(struct wrap_device *wd)
 		if (wrap_urb->state == URB_SUBMITTED) {
 			WARNING("Windows driver %s didn't free urb: %p",
 				wd->driver->name, wrap_urb->urb);
+			if (!complete)
+				wrap_urb->urb->complete = NULL;
 			usb_kill_urb(wrap_urb->urb);
 		}
 		usb_free_urb(wrap_urb->urb);
 		kfree(wrap_urb);
 	}
 	wd->usb.num_alloc_urbs = 0;
+}
+
+void usb_exit_device(struct wrap_device *wd)
+{
+	kill_all_urbs(wd, 0);
 	USBEXIT(return);
 }
 
@@ -249,8 +273,7 @@ static void wrap_free_urb(struct urb *urb)
 	USBTRACE("freeing urb: %p", urb);
 	wrap_urb = urb->context;
 	irp = wrap_urb->irp;
-	if (urb->transfer_buffer &&
-	    (wrap_urb->alloc_flags & URB_NO_TRANSFER_DMA_MAP)) {
+	if (wrap_urb->flags & WRAP_URB_COPY_BUFFER) {
 		USBTRACE("freeing DMA buffer for URB: %p %p",
 			 urb, urb->transfer_buffer);
 		usb_buffer_free(irp->wd->usb.udev,
@@ -268,7 +291,7 @@ static void wrap_free_urb(struct urb *urb)
 		kfree(wrap_urb);
 	} else {
 		wrap_urb->state = URB_FREE;
-		wrap_urb->alloc_flags = 0;
+		wrap_urb->flags = 0;
 		wrap_urb->irp = NULL;
 	}
 	return;
@@ -303,8 +326,8 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 	IoAcquireCancelSpinLock(&irp->cancel_irql);
 	urb = NULL;
 	nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list, list) {
-		if (wrap_urb->state == URB_FREE) {
-			wrap_urb->state = URB_ALLOCATED;
+		if (cmpxchg(&wrap_urb->state, URB_FREE,
+			    URB_ALLOCATED) == URB_FREE) {
 			urb = wrap_urb->urb;
 			usb_init_urb(urb);
 			break;
@@ -337,10 +360,11 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 
 #ifdef URB_ASYNC_UNLINK
 	urb->transfer_flags |= URB_ASYNC_UNLINK;
+#elif defined(USB_ASYNC_UNLINK)
+	urb->transfer_flags |= USB_ASYNC_UNLINK;
 #endif
 	urb->context = wrap_urb;
 	wrap_urb->irp = irp;
-	wrap_urb->pipe = pipe;
 	irp->wrap_urb = wrap_urb;
 	irp->cancel_routine = wrap_cancel_irp;
 	IoReleaseCancelSpinLock(irp->cancel_irql);
@@ -364,8 +388,9 @@ static struct urb *wrap_alloc_urb(struct irp *irp, unsigned int pipe,
 			IoReleaseCancelSpinLock(irp->cancel_irql);
 			return NULL;
 		}
-		urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;
-		wrap_urb->alloc_flags |= URB_NO_TRANSFER_DMA_MAP;
+		if (urb->transfer_dma)
+			urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;
+		wrap_urb->flags |= WRAP_URB_COPY_BUFFER;
 		if (usb_pipeout(pipe))
 			memcpy(urb->transfer_buffer, buf, buf_len);
 		USBTRACE("DMA buffer for urb %p is %p",
@@ -424,47 +449,67 @@ static USBD_STATUS wrap_submit_urb(struct irp *irp)
 		USBEXIT(return USBD_STATUS_PENDING);
 }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-static void wrap_urb_complete(struct urb *urb, struct pt_regs *regs)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+static void int_urb_unlink_complete(struct urb *urb)
+{
+	struct wrap_urb *wrap_urb;
+
+	wrap_urb = urb->context;
+	USBTRACE("%p, %d, %d", urb, urb->status, wrap_urb->state);
+	if (xchg(&wrap_urb->state, URB_INT_UNLINKED) != URB_COMPLETED)
+		return;
+	nt_spin_lock(&wrap_urb_complete_list_lock);
+	InsertTailList(&wrap_urb_complete_list, &wrap_urb->complete_list);
+	nt_spin_unlock(&wrap_urb_complete_list_lock);
+#ifdef USB_TASKLET
+	tasklet_schedule(&wrap_urb_complete_work);
 #else
+	schedule_ntos_work(&wrap_urb_complete_work);
+#endif
+}
+
 static void wrap_urb_complete(struct urb *urb)
+#else
+static void wrap_urb_complete(struct urb *urb, struct pt_regs *regs)
 #endif
 {
 	struct irp *irp;
 	struct wrap_urb *wrap_urb;
 
 	wrap_urb = urb->context;
+	USBTRACE("%p (%p) completed", wrap_urb, urb);
 	irp = wrap_urb->irp;
 	DUMP_WRAP_URB(wrap_urb, USB_DIR_IN);
 	irp->cancel_routine = NULL;
-	if (wrap_urb->state == URB_SUBMITTED ||
-	    wrap_urb->state == URB_CANCELED) {
-		wrap_urb->state = URB_COMPLETED;
-		/* Prevent 2.4 kernels from resubmiting interrupt
-		 * URBs.  UHCI/OHCI don't resubmit if URB interval is
-		 * 0 and EHCI doesn't resubmit if urb->status is
-		 * -ENOENT/-ECONNRESET */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-		URB_STATUS(wrap_urb) = urb->status;
-		if (usb_pipeint(urb->pipe)) {
-			urb->interval = 0;
-			urb->status = -ENOENT;
-		}
-#endif
-		nt_spin_lock(&wrap_urb_complete_list_lock);
-		InsertTailList(&wrap_urb_complete_list,
-			       &wrap_urb->complete_list);
-		nt_spin_unlock(&wrap_urb_complete_list_lock);
-#ifdef USB_TASKLET
-		tasklet_schedule(&wrap_urb_complete_work);
-#else
-		schedule_work(&wrap_urb_complete_work);
-#endif
-		USBTRACE("scheduled worker for urb %p", urb);
-	} else {
-		WARNING("urb %p in wrong state: %d", urb, wrap_urb->state);
+#ifdef USB_DEBUG
+	if (wrap_urb->state != URB_SUBMITTED &&
+	    wrap_urb->state != URB_CANCELED) {
+		WARNING("urb %p in wrong state: %d (%d)", urb, wrap_urb->state,
+			urb->status);
+		return;
 	}
-	USBTRACE("urb %p (irp: %p) completed", urb, irp);
+#endif
+	wrap_urb->state = URB_COMPLETED;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+	URB_STATUS(wrap_urb) = urb->status;
+	/* prevent 2.4 kernels from resubmitting interrupt urbs;
+	 * completion function for unlink will process urb */
+	if (usb_pipeint(urb->pipe)) {
+		USBTRACE("%p, %d", urb, urb->status);
+		urb->complete = int_urb_unlink_complete;
+		usb_unlink_urb(urb);
+		return;
+	}
+#endif
+	nt_spin_lock(&wrap_urb_complete_list_lock);
+	InsertTailList(&wrap_urb_complete_list, &wrap_urb->complete_list);
+	nt_spin_unlock(&wrap_urb_complete_list_lock);
+#ifdef USB_TASKLET
+	tasklet_schedule(&wrap_urb_complete_work);
+#else
+	schedule_ntos_work(&wrap_urb_complete_work);
+#endif
+	USBTRACE("scheduled worker for urb %p", urb);
 }
 
 /* one worker for all devices */
@@ -492,12 +537,15 @@ static void wrap_urb_complete_worker(void *dummy)
 			break;
 		wrap_urb = container_of(ent, struct wrap_urb, complete_list);
 		urb = wrap_urb->urb;
+#ifdef USB_DEBUG
+		if (wrap_urb->state != URB_COMPLETED &&
+		    wrap_urb->state != URB_INT_UNLINKED)
+			WARNING("urb %p in wrong state: %d",
+				urb, wrap_urb->state);
+#endif
 		irp = wrap_urb->irp;
 		DUMP_IRP(irp);
 		nt_urb = URB_FROM_IRP(irp);
-		if (wrap_urb->state != URB_COMPLETED)
-			WARNING("urb %p in wrong state: %d",
-				urb, wrap_urb->state);
 		USBTRACE("urb: %p, nt_urb: %p, status: %d",
 			 urb, nt_urb, URB_STATUS(wrap_urb));
 		switch (URB_STATUS(wrap_urb)) {
@@ -510,8 +558,7 @@ static void wrap_urb_complete_worker(void *dummy)
 				bulk_int_tx->transfer_buffer_length =
 					urb->actual_length;
 				DUMP_URB_BUFFER(urb, USB_DIR_IN);
-				if ((wrap_urb->alloc_flags &
-				     URB_NO_TRANSFER_DMA_MAP) &&
+				if ((wrap_urb->flags & WRAP_URB_COPY_BUFFER) &&
 				    usb_pipein(urb->pipe))
 					memcpy(bulk_int_tx->transfer_buffer,
 					       urb->transfer_buffer,
@@ -521,8 +568,7 @@ static void wrap_urb_complete_worker(void *dummy)
 				vc_req->transfer_buffer_length =
 					urb->actual_length;
 				DUMP_URB_BUFFER(urb, USB_DIR_IN);
-				if ((wrap_urb->alloc_flags &
-				     URB_NO_TRANSFER_DMA_MAP) &&
+				if ((wrap_urb->flags & WRAP_URB_COPY_BUFFER) &&
 				    usb_pipein(urb->pipe))
 					memcpy(vc_req->transfer_buffer,
 					       urb->transfer_buffer,
@@ -552,7 +598,7 @@ static void wrap_urb_complete_worker(void *dummy)
 
 /* called as Windows function, so call WIN2LIN2 before accessing
  * arguments */
-static STDCALL void wrap_cancel_irp(struct device_object *dev_obj,
+static wstdcall void wrap_cancel_irp(struct device_object *dev_obj,
 				    struct irp *irp)
 {
 	struct urb *urb;
@@ -563,13 +609,11 @@ static STDCALL void wrap_cancel_irp(struct device_object *dev_obj,
 	USBENTER("irp: %p", irp);
 	urb = irp->wrap_urb->urb;
 	USBTRACE("canceling urb %p", urb);
-	if (irp->wrap_urb->state == URB_SUBMITTED &&
-	    wrap_cancel_urb(irp->wrap_urb) == 0) {
+	if (wrap_cancel_urb(irp->wrap_urb) == 0) {
 		USBTRACE("urb %p canceled", urb);
 		/* this IRP will be returned in urb's completion function */
 	} else
-		ERROR("urb %p can't be canceld: %d",
-		      urb, irp->wrap_urb->state);
+		ERROR("urb %p can't be canceld: %d", urb, irp->wrap_urb->state);
 	IoReleaseCancelSpinLock(irp->cancel_irql);
 	return;
 }
@@ -634,8 +678,7 @@ static USBD_STATUS wrap_bulk_or_intr_trans(struct irp *irp)
 				 pipe_handle->bInterval);
 		USBTRACE("submitting interrupt urb %p on pipe 0x%x (ep 0x%x), "
 			 "intvl: %d", urb, urb->pipe,
-			 pipe_handle->bEndpointAddress,
-			 pipe_handle->bInterval);
+			 pipe_handle->bEndpointAddress, pipe_handle->bInterval);
 	}
 	status = wrap_submit_urb(irp);
 	USBTRACE("status: %08X", status);
@@ -790,31 +833,31 @@ static USBD_STATUS wrap_abort_pipe(struct usb_device *udev, struct irp *irp)
 {
 	union nt_urb *nt_urb;
 	usbd_pipe_handle pipe_handle;
-	struct urb *urb;
 	struct wrap_urb *wrap_urb;
 	struct wrap_device *wd;
 	KIRQL irql;
 
-	USBENTER("irp = %p", irp);
 	wd = irp->wd;
 	nt_urb = URB_FROM_IRP(irp);
 	pipe_handle = nt_urb->pipe_req.pipe_handle;
+	USBENTER("%p, %x", irp, pipe_handle->bEndpointAddress);
 	nt_urb = URB_FROM_IRP(irp);
 	IoAcquireCancelSpinLock(&irql);
 	nt_list_for_each_entry(wrap_urb, &wd->usb.wrap_urb_list, list) {
-		if (wrap_urb->state == URB_SUBMITTED &&
-		    (usb_pipeendpoint(wrap_urb->pipe) ==
-		     pipe_handle->bEndpointAddress)) {
-			urb = wrap_urb->urb;
-			wrap_cancel_urb(wrap_urb);
-			USBTRACE("canceled urb: %p", urb);
+		USBTRACE("%p, %p, %d, %x, %x", wrap_urb, wrap_urb->urb,
+			 wrap_urb->state, wrap_urb->urb->pipe,
+			 usb_pipeendpoint(wrap_urb->urb->pipe));
+		/* for WG111T driver, urbs for endpoint 0 should also
+		 * be canceled */
+		if ((usb_pipeendpoint(wrap_urb->urb->pipe) ==
+		     pipe_handle->bEndpointAddress) ||
+		    (usb_pipeendpoint(wrap_urb->urb->pipe) == 0)) {
+			if (wrap_cancel_urb(wrap_urb) == 0)
+				USBTRACE("canceled wrap_urb: %p", wrap_urb);
 		}
 	}
 	IoReleaseCancelSpinLock(irql);
-	if (wd->usb.intf == NULL)
-		USBEXIT(return USBD_STATUS_DEVICE_GONE);
-	else
-		USBEXIT(return USBD_STATUS_SUCCESS);
+	USBEXIT(return USBD_STATUS_SUCCESS);
 }
 
 static void set_intf_pipe_info(struct wrap_device *wd,
@@ -885,13 +928,23 @@ static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 	if (config == NULL) {
 		/* TODO: set to unconfigured state (configuration 0):
 		 * is this correctt? */
+#if 0
+		ret = usb_control_msg(udev, usb_sndctrlpipe(udev, 0),
+				      USB_REQ_SET_CONFIGURATION, 0,
+				      0, 0, NULL, 0, USB_CTRL_SET_TIMEOUT);
+		ret = usb_reset_configuration(udev);
+		usb_disable_device(udev, 0);
+#endif
+		kill_all_urbs(wd, 1);
 		ret = usb_control_msg(udev, usb_sndctrlpipe(udev, 0),
 				      USB_REQ_SET_CONFIGURATION, 0,
 				      0, 0, NULL, 0, USB_CTRL_SET_TIMEOUT);
 		USBTRACE("ret: %d", ret);
+#if 0
 		if (ret < 0)
 			return wrap_urb_status(ret);
 		else
+#endif
 			return USBD_STATUS_SUCCESS;
 	}
 
@@ -904,7 +957,6 @@ static USBD_STATUS wrap_select_configuration(struct wrap_device *wd,
 			      USB_REQ_SET_CONFIGURATION, 0,
 			      config->bConfigurationValue, 0,
 			      NULL, 0, USB_CTRL_SET_TIMEOUT);
-	USBTRACE("ret: %d", ret);
 	if (ret < 0) {
 		ERROR("ret: %d", ret);
 		return wrap_urb_status(ret);
@@ -1095,7 +1147,7 @@ static USBD_STATUS wrap_process_nt_urb(struct irp *irp)
 
 static USBD_STATUS wrap_reset_port(struct irp *irp)
 {
-	int ret, lock = 0;
+	no_warn_unused int ret, lock = 0;
 	struct wrap_device *wd;
 
 	wd = irp->wd;
@@ -1198,7 +1250,7 @@ NTSTATUS wrap_submit_irp(struct device_object *pdo, struct irp *irp)
  * description and examples elsewhere suggest that it should be
  * usbd_interface_list_entry structre. Which is correct? */
 
-STDCALL union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequestEx)
+wstdcall union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequestEx)
 	(struct usb_config_descriptor *config,
 	 struct usbd_interface_list_entry *intf_list)
 {
@@ -1263,7 +1315,7 @@ STDCALL union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequestEx)
 
 WRAP_EXPORT_MAP("_USBD_CreateConfigurationRequestEx@8",	USBD_CreateConfigurationRequestEx);
 
-STDCALL struct usb_interface_descriptor *
+wstdcall struct usb_interface_descriptor *
 WRAP_EXPORT(USBD_ParseConfigurationDescriptorEx)
 	(struct usb_config_descriptor *config, void *start,
 	 LONG bInterfaceNumber, LONG bAlternateSetting, LONG bInterfaceClass,
@@ -1302,7 +1354,7 @@ WRAP_EXPORT(USBD_ParseConfigurationDescriptorEx)
 
 WRAP_EXPORT_MAP("_USBD_ParseConfigurationDescriptorEx@28", USBD_ParseConfigurationDescriptorEx);
 
-STDCALL union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequest)
+wstdcall union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequest)
 	(struct usb_config_descriptor *config, USHORT *size)
 {
 	union nt_urb *nt_urb;
@@ -1325,7 +1377,7 @@ STDCALL union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequest)
 	USBEXIT(return nt_urb);
 }
 
-STDCALL struct usb_interface_descriptor *
+wstdcall struct usb_interface_descriptor *
 WRAP_EXPORT(USBD_ParseConfigurationDescriptor)
 	(struct usb_config_descriptor *config, UCHAR bInterfaceNumber,
 	 UCHAR bAlternateSetting)
@@ -1336,7 +1388,7 @@ WRAP_EXPORT(USBD_ParseConfigurationDescriptor)
 						   -1, -1, -1);
 }
 
-STDCALL void WRAP_EXPORT(USBD_GetUSBDIVersion)
+wstdcall void WRAP_EXPORT(USBD_GetUSBDIVersion)
 	(struct usbd_version_info *version_info)
 {
 	/* this function is obsolete in Windows XP */
@@ -1348,7 +1400,7 @@ STDCALL void WRAP_EXPORT(USBD_GetUSBDIVersion)
 	USBEXIT(return);
 }
 
-STDCALL void
+wstdcall void
 USBD_InterfaceGetUSBDIVersion(void *context,
 			      struct usbd_version_info *version_info,
 			      ULONG *hcd_capa)
@@ -1366,7 +1418,7 @@ USBD_InterfaceGetUSBDIVersion(void *context,
 	USBEXIT(return);
 }
 
-STDCALL BOOLEAN USBD_InterfaceIsDeviceHighSpeed(void *context)
+wstdcall BOOLEAN USBD_InterfaceIsDeviceHighSpeed(void *context)
 {
 	struct wrap_device *wd = context;
 
@@ -1377,19 +1429,19 @@ STDCALL BOOLEAN USBD_InterfaceIsDeviceHighSpeed(void *context)
 		USBEXIT(return FALSE);
 }
 
-STDCALL void USBD_InterfaceReference(void *context)
+wstdcall void USBD_InterfaceReference(void *context)
 {
 	USBTRACE("%p", context);
 	UNIMPL();
 }
 
-STDCALL void USBD_InterfaceDereference(void *context)
+wstdcall void USBD_InterfaceDereference(void *context)
 {
 	USBTRACE("%p", context);
 	UNIMPL();
 }
 
-STDCALL NTSTATUS USBD_InterfaceQueryBusTime(void *context, ULONG *frame)
+wstdcall NTSTATUS USBD_InterfaceQueryBusTime(void *context, ULONG *frame)
 {
 	struct wrap_device *wd = context;
 
@@ -1397,7 +1449,7 @@ STDCALL NTSTATUS USBD_InterfaceQueryBusTime(void *context, ULONG *frame)
 	USBEXIT(return STATUS_SUCCESS);
 }
 
-STDCALL NTSTATUS USBD_InterfaceSubmitIsoOutUrb(void *context,
+wstdcall NTSTATUS USBD_InterfaceSubmitIsoOutUrb(void *context,
 					       union nt_urb *nt_urb)
 {
 	/* TODO: implement this */
@@ -1405,7 +1457,7 @@ STDCALL NTSTATUS USBD_InterfaceSubmitIsoOutUrb(void *context,
 	USBEXIT(return STATUS_NOT_IMPLEMENTED);
 }
 
-STDCALL NTSTATUS
+wstdcall NTSTATUS
 USBD_InterfaceQueryBusInformation(void *context, ULONG level, void *buf,
 				  ULONG *buf_length, ULONG *buf_actual_length)
 {
@@ -1419,7 +1471,7 @@ USBD_InterfaceQueryBusInformation(void *context, ULONG level, void *buf,
 	USBEXIT(return STATUS_NOT_IMPLEMENTED);
 }
 
-STDCALL NTSTATUS
+wstdcall NTSTATUS
 USBD_InterfaceLogEntry(void *context, ULONG driver_tag, ULONG enum_tag,
 		       ULONG p1, ULONG p2)
 {
