@@ -19,13 +19,8 @@
 #include "loader.h"
 #include "wrapndis.h"
 
-extern NT_SPIN_LOCK loader_lock;
-extern struct wrap_device *wrap_devices;
-extern struct nt_list wrap_drivers;
-
 extern char *if_name;
 extern int hangcheck_interval;
-extern int disable_stats;
 extern struct iw_handler_def ndis_handler_def;
 
 static int set_packet_filter(struct wrap_ndis_device *wnd,
@@ -34,8 +29,6 @@ static void add_stats_timer(struct wrap_ndis_device *wnd);
 static void del_stats_timer(struct wrap_ndis_device *wnd);
 static NDIS_STATUS ndis_start_device(struct wrap_ndis_device *wnd);
 static int ndis_remove_device(struct wrap_ndis_device *wnd);
-static NDIS_STATUS miniport_set_power_state(struct wrap_ndis_device *wnd,
-					    enum ndis_power_state state);
 static void set_multicast_list(struct wrap_ndis_device *wnd);
 
 static inline int ndis_wait_comm_completion(struct wrap_ndis_device *wnd)
@@ -50,7 +43,7 @@ static inline int ndis_wait_comm_completion(struct wrap_ndis_device *wnd)
 /* MiniportReset */
 NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 {
-	NDIS_STATUS res = 0;
+	NDIS_STATUS res;
 	struct miniport_char *miniport;
 	UINT cur_lookahead, max_lookahead;
 	BOOLEAN reset_address;
@@ -58,21 +51,18 @@ NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 
 	TRACEENTER2("wnd: %p", wnd);
 
-	if (!test_bit(HW_AVAILABLE, &wnd->hw_status))
-		TRACEEXIT1(return NDIS_STATUS_FAILURE);
-
 	if (down_interruptible(&wnd->ndis_comm_mutex))
 		TRACEEXIT3(return NDIS_STATUS_FAILURE);
+	down_interruptible(&wnd->tx_ring_mutex);
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	cur_lookahead = wnd->nmb->cur_lookahead;
 	max_lookahead = wnd->nmb->max_lookahead;
 	irql = raise_irql(DISPATCH_LEVEL);
 	wnd->ndis_comm_done = 0;
-	res = LIN2WIN2(miniport->reset, &reset_address,
-		       wnd->nmb->adapter_ctx);
+	res = LIN2WIN2(miniport->reset, &reset_address, wnd->nmb->adapter_ctx);
 	lower_irql(irql);
 
-	DBGTRACE2("res = %08X, reset_status = %08X", res, reset_address);
+	DBGTRACE2("%08X, %08X", res, reset_address);
 	if (res == NDIS_STATUS_PENDING) {
 		/* wait for NdisMResetComplete */
 		if (ndis_wait_comm_completion(wnd))
@@ -81,11 +71,8 @@ NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 			res = wnd->ndis_comm_status;
 			reset_address = wnd->ndis_comm_done - 1;
 		}
-		DBGTRACE2("res = %08X, reset_status = %08X",
-			  res, reset_address);
+		DBGTRACE2("%08X, %08X", res, reset_address);
 	}
-	DBGTRACE2("reset: res = %08X, reset status = %08X",
-		  res, reset_address);
 	up(&wnd->ndis_comm_mutex);
 	if (res == NDIS_STATUS_SUCCESS && reset_address) {
 		wnd->nmb->cur_lookahead = cur_lookahead;
@@ -93,7 +80,7 @@ NDIS_STATUS miniport_reset(struct wrap_ndis_device *wnd)
 		set_packet_filter(wnd, wnd->packet_filter);
 		set_multicast_list(wnd);
 	}
-
+	up(&wnd->tx_ring_mutex);
 	TRACEEXIT3(return res);
 }
 
@@ -102,39 +89,38 @@ NDIS_STATUS miniport_query_info_needed(struct wrap_ndis_device *wnd,
 				       ndis_oid oid, void *buf,
 				       ULONG bufsize, ULONG *needed)
 {
-	NDIS_STATUS res = 0;
+	NDIS_STATUS res;
 	ULONG written;
 	struct miniport_char *miniport;
 	KIRQL irql;
 
 	DBGTRACE2("oid: %08X", oid);
 
-	if (!test_bit(HW_AVAILABLE, &wnd->hw_status))
-		TRACEEXIT1(return NDIS_STATUS_FAILURE);
-
 	if (down_interruptible(&wnd->ndis_comm_mutex))
 		TRACEEXIT3(return NDIS_STATUS_FAILURE);
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
-	DBGTRACE2("query %p, oid: %08X", miniport->query, oid);
+	DBGTRACE2("%p, %08X", miniport->query, oid);
 	irql = raise_irql(DISPATCH_LEVEL);
 	wnd->ndis_comm_done = 0;
 	res = LIN2WIN6(miniport->query, wnd->nmb->adapter_ctx, oid, buf,
 		       bufsize, &written, needed);
 	lower_irql(irql);
 
-	DBGTRACE2("res: %08X, oid: %08X", res, oid);
+	DBGTRACE2("%08X, %08X", res, oid);
 	if (res == NDIS_STATUS_PENDING) {
 		/* wait for NdisMQueryInformationComplete */
 		if (ndis_wait_comm_completion(wnd))
 			res = NDIS_STATUS_FAILURE;
 		else
 			res = wnd->ndis_comm_status;
-		DBGTRACE2("res: %08X, oid: %08X", res, oid);
+		DBGTRACE2("%08X, %08X", res, oid);
 	}
 	up(&wnd->ndis_comm_mutex);
-	if (res || needed)
-		DBGTRACE2("res: %08X, bufsize: %d, written: %d, needed: %d",
-			  res, bufsize, written, *needed);
+	DBG_BLOCK(2) {
+		if (res || needed)
+			DBGTRACE2("%08X, %d, %d, %d", res, bufsize, written,
+				  *needed);
+	}
 	TRACEEXIT3(return res);
 }
 
@@ -152,40 +138,38 @@ NDIS_STATUS miniport_query_info(struct wrap_ndis_device *wnd, ndis_oid oid,
 NDIS_STATUS miniport_set_info(struct wrap_ndis_device *wnd, ndis_oid oid,
 			      void *buf, ULONG bufsize)
 {
-	NDIS_STATUS res = 0;
+	NDIS_STATUS res;
 	ULONG written, needed;
 	struct miniport_char *miniport;
 	KIRQL irql;
 
 	DBGTRACE2("oid: %08X", oid);
 
-	if (!test_bit(HW_AVAILABLE, &wnd->hw_status))
-		TRACEEXIT1(return NDIS_STATUS_FAILURE);
-
 	if (down_interruptible(&wnd->ndis_comm_mutex))
 		TRACEEXIT3(return NDIS_STATUS_FAILURE);
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
-	DBGTRACE2("query %p, oid: %08X", miniport->query, oid);
+	DBGTRACE2("%p, %08X", miniport->query, oid);
 	irql = raise_irql(DISPATCH_LEVEL);
 	wnd->ndis_comm_done = 0;
 	res = LIN2WIN6(miniport->setinfo, wnd->nmb->adapter_ctx, oid,
 		       buf, bufsize, &written, &needed);
 	lower_irql(irql);
 
-	DBGTRACE2("res: %08X, oid: %08X", res, oid);
+	DBGTRACE2("%08X, %08X", res, oid);
 	if (res == NDIS_STATUS_PENDING) {
 		/* wait for NdisMQueryInformationComplete */
 		if (ndis_wait_comm_completion(wnd))
 			res = NDIS_STATUS_FAILURE;
 		else
 			res = wnd->ndis_comm_status;
-		DBGTRACE2("res: %08X, oid: %08X", res, oid);
+		DBGTRACE2("%08X, %08X", res, oid);
 	}
 	up(&wnd->ndis_comm_mutex);
-
-	if (res && needed)
-		DBGTRACE2("res: %08X, bufsize: %d, written: %d, needed: %d",
-			  res, bufsize, written, needed);
+	DBG_BLOCK(2) {
+		if (res && needed)
+			DBGTRACE2("%08X, %d, %d, %d", res, bufsize, written,
+				  needed);
+	}
 	TRACEEXIT3(return res);
 }
 
@@ -238,7 +222,6 @@ static NDIS_STATUS miniport_pnp_event(struct wrap_ndis_device *wnd,
 			return NDIS_STATUS_FAILURE;
 		}
 		power_profile = NdisPowerProfileAcOnLine;
-		DBGTRACE2("calling pnp_event_notify");
 		LIN2WIN4(miniport->pnp_event_notify, wnd->nmb->adapter_ctx,
 			 NdisDevicePnPEventPowerProfileChanged,
 			 &power_profile, (ULONG)sizeof(power_profile));
@@ -260,13 +243,13 @@ static NDIS_STATUS miniport_init(struct wrap_ndis_device *wnd)
 
 	TRACEENTER1("irql: %d", current_irql());
 	if (test_bit(HW_INITIALIZED, &wnd->hw_status)) {
-		ERROR("device %p already initialized!", wnd);
+		WARNING("device %p already initialized!", wnd);
 		return NDIS_STATUS_FAILURE;
 	}
 
 	if (!wnd->wd->driver->ndis_driver ||
 	    !wnd->wd->driver->ndis_driver->miniport.init) {
-		ERROR("assuming WDM (non-NDIS) driver");
+		WARNING("assuming WDM (non-NDIS) driver");
 		TRACEEXIT1(return NDIS_STATUS_NOT_RECOGNIZED);
 	}
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
@@ -281,17 +264,21 @@ static NDIS_STATUS miniport_init(struct wrap_ndis_device *wnd)
 	}
 
 	/* Wait a little to let card power up otherwise ifup might
-	 * fail after boot; USB devices seem to need long delays */
-	set_current_state(TASK_INTERRUPTIBLE);
-	schedule_timeout(1 * HZ);
-
+	 * fail after boot */
+	sleep_hz(HZ / 2);
 	set_bit(HW_INITIALIZED, &wnd->hw_status);
-	set_bit(HW_AVAILABLE, &wnd->hw_status);
+	hangcheck_add(wnd);
+	up(&wnd->ndis_comm_mutex);
+	up(&wnd->tx_ring_mutex);
+	/* the description about NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND is
+	 * misleading/confusing; we just ignore it */
 	status = miniport_query_info(wnd, OID_PNP_CAPABILITIES,
 				     &pnp_capa, sizeof(pnp_capa));
-	if (status != NDIS_STATUS_SUCCESS)
-		wnd->attributes &= ~NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND;
-	hangcheck_add(wnd);
+	if (status == NDIS_STATUS_SUCCESS)
+		wnd->pm_capa = TRUE;
+	else
+		wnd->pm_capa = FALSE;
+	DBGTRACE1("%d", pnp_capa.wakeup_capa.min_magic_packet_wakeup);
 	TRACEEXIT1(return NDIS_STATUS_SUCCESS);
 }
 
@@ -301,18 +288,19 @@ static void miniport_halt(struct wrap_ndis_device *wnd)
 	struct miniport_char *miniport;
 
 	TRACEENTER1("%p", wnd);
-	clear_bit(HW_AVAILABLE, &wnd->hw_status);
-	hangcheck_del(wnd);
-	del_stats_timer(wnd);
-	if (!test_bit(HW_INITIALIZED, &wnd->hw_status)) {
+	/* semaphores may already be locked, e.g., during suspend or
+	 * if device is suspended, but resume failed */
+	down_trylock(&wnd->ndis_comm_mutex);
+	down_trylock(&wnd->tx_ring_mutex);
+	if (test_bit(HW_INITIALIZED, &wnd->hw_status)) {
+		hangcheck_del(wnd);
+		del_stats_timer(wnd);
+		miniport = &wnd->wd->driver->ndis_driver->miniport;
+		DBGTRACE1("halt: %p", miniport->miniport_halt);
+		LIN2WIN1(miniport->miniport_halt, wnd->nmb->adapter_ctx);
+		clear_bit(HW_INITIALIZED, &wnd->hw_status);
+	} else
 		WARNING("device %p is not initialized - not halting", wnd);
-		TRACEEXIT1(return);
-	}
-	miniport = &wnd->wd->driver->ndis_driver->miniport;
-	DBGTRACE1("driver halt is at %p", miniport->miniport_halt);
-	LIN2WIN1(miniport->miniport_halt, wnd->nmb->adapter_ctx);
-	clear_bit(HW_INITIALIZED, &wnd->hw_status);
-
 	TRACEEXIT1(return);
 }
 
@@ -321,30 +309,34 @@ static NDIS_STATUS miniport_set_power_state(struct wrap_ndis_device *wnd,
 {
 	NDIS_STATUS status;
 
-	DBGTRACE1("state: %d", state);
+	DBGTRACE1("%d", state);
 	if (state == NdisDeviceStateD0) {
-		status = STATUS_SUCCESS;
+		status = NDIS_STATUS_SUCCESS;
 		if (test_and_clear_bit(HW_HALTED, &wnd->hw_status)) {
-			DBGTRACE1("starting device");
-			/* sis163u driver crashes when resuming from
-			 * s3; USB devices will be restarted by USB
-			 * driver anyway, so for now just ignore  */
-			if (wrap_is_usb_bus(wnd->wd->dev_bus_type))
-				status = NDIS_STATUS_SUCCESS;
-			else
-				status = miniport_init(wnd);
+			status = miniport_init(wnd);
+			if (status == NDIS_STATUS_SUCCESS) {
+				set_packet_filter(wnd, wnd->packet_filter);
+				set_multicast_list(wnd);
+			}
 		} else if (test_and_clear_bit(HW_SUSPENDED, &wnd->hw_status)) {
-			status = miniport_set_info(wnd, OID_PNP_SET_POWER,
-						   &state, (ULONG)sizeof(state));
-			if (status != NDIS_STATUS_SUCCESS)
+			up(&wnd->ndis_comm_mutex);
+			status = miniport_set_int(wnd, OID_PNP_SET_POWER,
+						  state);
+			if (status == NDIS_STATUS_SUCCESS)
+				up(&wnd->tx_ring_mutex);
+			else {
+				down_interruptible(&wnd->ndis_comm_mutex);
 				WARNING("%s: setting power to state %d failed? "
 					"%08X", wnd->net_dev->name, state,
 					status);
-			status = NDIS_STATUS_SUCCESS;
-		}
+			}
+			if (wnd->ndis_wolopts &&
+			    wrap_is_pci_bus(wnd->wd->dev_bus_type))
+				pci_enable_wake(wnd->wd->pci.pdev, PCI_D0, 0);
+		} else
+			return NDIS_STATUS_FAILURE;
+
 		if (status == NDIS_STATUS_SUCCESS) {
-			set_packet_filter(wnd, wnd->packet_filter);
-			set_multicast_list(wnd);
 			hangcheck_add(wnd);
 			add_stats_timer(wnd);
 			set_scan(wnd);
@@ -353,36 +345,44 @@ static NDIS_STATUS miniport_set_power_state(struct wrap_ndis_device *wnd,
 				netif_wake_queue(wnd->net_dev);
 			}
 			netif_poll_enable(wnd->net_dev);
-		} else
+		} else {
 			WARNING("%s: couldn't set power to state %d; device not"
 				" resumed", wnd->net_dev->name, state);
+		}
 		TRACEEXIT1(return status);
 	} else {
-		status = NDIS_STATUS_NOT_SUPPORTED;
-		DBGTRACE2("irql: %d", current_irql());
 		netif_poll_disable(wnd->net_dev);
 		if (netif_running(wnd->net_dev)) {
 			netif_tx_disable(wnd->net_dev);
 			netif_device_detach(wnd->net_dev);
 		}
+		if (down_interruptible(&wnd->tx_ring_mutex))
+			WARNING("couldn't lock tx_ring_mutex");
 		hangcheck_del(wnd);
 		del_stats_timer(wnd);
-		if (wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND) {
+		status = NDIS_STATUS_NOT_SUPPORTED;
+		if (wnd->pm_capa == TRUE) {
+			enum ndis_power_state pm_state = state;
 			if (wnd->ndis_wolopts) {
 				status = miniport_set_int(wnd,
 							  OID_PNP_ENABLE_WAKE_UP,
 							  wnd->ndis_wolopts);
-				if (status != NDIS_STATUS_SUCCESS)
+				if (status == NDIS_STATUS_SUCCESS) {
+					if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
+						pci_enable_wake(wnd->wd->pci.pdev,
+								PCI_D0, 1);
+				} else
 					WARNING("%s: couldn't enable WOL: %08x",
 						wnd->net_dev->name, status);
 			}
-			status = miniport_set_info(wnd, OID_PNP_SET_POWER,
-						   &state, (ULONG)sizeof(state));
-			if (status == NDIS_STATUS_SUCCESS)
+			status = miniport_set_int(wnd, OID_PNP_SET_POWER,
+						  pm_state);
+			if (status == NDIS_STATUS_SUCCESS) {
 				set_bit(HW_SUSPENDED, &wnd->hw_status);
-			else
-				WARNING("setting power state to %d failed: %08X",
-					state, status);
+				if (down_interruptible(&wnd->ndis_comm_mutex))
+					WARNING("couldn't lock ndis_comm_mutex");
+			} else
+				WARNING("suspend failed: %08X", status);
 		}
 		if (status != NDIS_STATUS_SUCCESS) {
 			WARNING("%s does not support power management; "
@@ -413,7 +413,6 @@ static int ndis_set_mac_addr(struct net_device *dev, void *p)
 	memset(mac_string, 0, sizeof(mac_string));
 	res = snprintf(mac_string, sizeof(mac_string), MACSTR,
 		       MAC2STR(mac));
-	DBGTRACE2("res = %d, mac_tring = %s", res, mac_string);
 	if (res != (sizeof(mac_string) - 1))
 		TRACEEXIT1(return -EINVAL);
 
@@ -573,7 +572,6 @@ static int miniport_tx_packets(struct wrap_ndis_device *wnd)
 					/* resubmit this packet and
 					 * the rest when resources
 					 * become available */
-					netif_stop_queue(wnd->net_dev);
 					sent--;
 					break;
 				case NDIS_STATUS_FAILURE:
@@ -599,30 +597,28 @@ static int miniport_tx_packets(struct wrap_ndis_device *wnd)
 		}
 		DBGTRACE3("sent: %d(%d)", sent, n);
 	} else {
-		packet = wnd->tx_ring[start];
-		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE)) {
+		int i;
+		irql = PASSIVE_LEVEL;
+		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
 			irql = raise_irql(DISPATCH_LEVEL);
+		for (i = 0; i < n; i++) {
+			packet = wnd->tx_ring[(start + i) % TX_RING_SIZE];
 			res = LIN2WIN3(miniport->send, wnd->nmb->adapter_ctx,
 				       packet, packet->private.flags);
-			lower_irql(irql);
-		} else
-			res = LIN2WIN3(miniport->send, wnd->nmb->adapter_ctx,
-				       packet, packet->private.flags);
-		sent = 1;
-		switch (res) {
-		case NDIS_STATUS_SUCCESS:
-			free_tx_packet(wnd, packet, res);
-			break;
-		case NDIS_STATUS_PENDING:
-			break;
-		case NDIS_STATUS_RESOURCES:
-			wnd->tx_ok = 0;
-			sent = 0;
-			break;
-		case NDIS_STATUS_FAILURE:
-			free_tx_packet(wnd, packet, res);
-			break;
+			if (res == NDIS_STATUS_SUCCESS) {
+				free_tx_packet(wnd, packet, res);
+				continue;
+			} else {
+				DBGTRACE3("%08X", res);
+				i--;
+				if (res == NDIS_STATUS_RESOURCES)
+					wnd->tx_ok = 0;
+				break;
+			}
 		}
+		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
+			lower_irql(irql);
+		sent = i + 1;
 	}
 	TRACEEXIT3(return sent);
 }
@@ -633,18 +629,13 @@ static void tx_worker(void *param)
 	int n;
 
 	TRACEENTER3("tx_ok %d", wnd->tx_ok);
-
-	/* we can use cmpxchg to update tx_ring_start, but we need to
-	 * send packets in given order, so serialize (even for
-	 * deserialized drivers) with mutex */
-
-	while (1) {
+	while (wnd->tx_ok) {
 		if (down_interruptible(&wnd->tx_ring_mutex))
 			break;
 		/* end == start if either ring is empty or full; in
 		 * the latter case is_tx_ring_full is set */
-		if ((wnd->tx_ring_end == wnd->tx_ring_start &&
-		     !wnd->is_tx_ring_full) || !wnd->tx_ok) {
+		if (wnd->tx_ring_end == wnd->tx_ring_start &&
+		    !wnd->is_tx_ring_full) {
 			up(&wnd->tx_ring_mutex);
 			break;
 		}
@@ -749,13 +740,6 @@ static int ndis_open(struct net_device *dev)
 		WARNING("couldn't set packet filter");
 		return -ENODEV;
 	}
-	packet_filter = wnd->packet_filter;
-	/* add any dev specific filters */
-	if (dev->flags & IFF_PROMISC)
-		packet_filter |= NDIS_PACKET_TYPE_PROMISCUOUS |
-			NDIS_PACKET_TYPE_ALL_LOCAL;
-	set_packet_filter(wnd, packet_filter);
-
 	netif_device_attach(dev);
 	netif_start_queue(dev);
 	return 0;
@@ -770,6 +754,17 @@ static int ndis_close(struct net_device *dev)
 	return 0;
 }
 
+#ifdef CONFIG_NET_POLL_CONTROLLER
+static void ndis_poll_controller(struct net_device *dev)
+{
+	struct wrap_ndis_device *wnd = netdev_priv(dev);
+
+	disable_irq(dev->irq);
+	ndis_isr(dev->irq, wnd, NULL);
+	enable_irq(dev->irq);
+}
+#endif
+
 /* this function is called fom BH context */
 static struct net_device_stats *ndis_get_stats(struct net_device *dev)
 {
@@ -804,9 +799,11 @@ static void ndis_get_drvinfo(struct net_device *dev,
 	if (wrap_is_pci_bus(wnd->wd->dev_bus_type))
 		strncpy(info->bus_info, pci_name(wnd->wd->pci.pdev),
 			sizeof(info->bus_info) - 1);
+#ifdef CONFIG_USB
 	else
 		usb_make_path(wnd->wd->usb.udev, info->bus_info,
 			      sizeof(info->bus_info) - 1);
+#endif
 	return;
 }
 
@@ -833,13 +830,15 @@ static int ndis_set_wol(struct net_device *dev, struct ethtool_wolinfo *wol)
 
 	if (!(wol->wolopts & WAKE_MAGIC))
 		return -EINVAL;
-	if (!(wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND))
+	if (!wnd->pm_capa)
 		return -EOPNOTSUPP;
 	status = miniport_query_info(wnd, OID_PNP_CAPABILITIES,
 				     &pnp_capa, sizeof(pnp_capa));
 	if (status != NDIS_STATUS_SUCCESS)
 		return -EOPNOTSUPP;
 	/* we always suspend to D3 */
+	DBGTRACE1("%d, %d", pnp_capa.wakeup_capa.min_magic_packet_wakeup,
+		  pnp_capa.wakeup_capa.min_pattern_wakeup);
 	if (pnp_capa.wakeup_capa.min_magic_packet_wakeup != NdisDeviceStateD3)
 		return -EOPNOTSUPP;
 	/* no other options supported */
@@ -950,7 +949,6 @@ static void set_multicast_list(struct wrap_ndis_device *wnd)
 static void link_status_handler(struct wrap_ndis_device *wnd)
 {
 	struct ndis_assoc_info *ndis_assoc_info;
-	unsigned char *assoc_info;
 	union iwreq_data wrqu;
 	NDIS_STATUS res;
 	const int assoc_size = sizeof(*ndis_assoc_info) + IW_CUSTOM_MAX + 32;
@@ -960,7 +958,7 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 	int i;
 #endif
 
-	TRACEENTER2("link status: %d", wnd->link_status);
+	TRACEENTER2("link: %d", wnd->link_status);
 	if (wnd->physical_medium != NdisPhysicalMediumWirelessLan)
 		TRACEEXIT2(return);
 	if (wnd->link_status == 0) {
@@ -972,37 +970,30 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 	}
 
 	wnd->tx_ok = 1;
-	assoc_info = kmalloc(assoc_size, GFP_KERNEL);
-	if (!assoc_info) {
+	ndis_assoc_info = kmalloc(assoc_size, GFP_KERNEL);
+	if (!ndis_assoc_info) {
 		ERROR("couldn't allocate memory");
 		TRACEEXIT2(return);
 	}
-	memset(assoc_info, 0, assoc_size);
+	memset(ndis_assoc_info, 0, assoc_size);
 
-	ndis_assoc_info = (struct ndis_assoc_info *)assoc_info;
 	res = miniport_query_info(wnd, OID_802_11_ASSOCIATION_INFORMATION,
-				  assoc_info, assoc_size);
+				  ndis_assoc_info, assoc_size);
 	if (res) {
 		DBGTRACE2("query assoc_info failed (%08X)", res);
-		kfree(assoc_info);
+		kfree(ndis_assoc_info);
 		TRACEEXIT2(return);
 	}
 
-	/*
-	 * TODO: backwards compatibility would require that IWEVCUSTOM
-	 * is sent even if WIRELESS_EXT > 17. This version does not do
-	 * this in order to allow wpa_supplicant to be tested with
-	 * WE-18.
-	 */
 #if WIRELESS_EXT > 17
 	memset(&wrqu, 0, sizeof(wrqu));
 	wrqu.data.length = ndis_assoc_info->req_ie_length;
 	wireless_send_event(wnd->net_dev, IWEVASSOCREQIE, &wrqu,
-			    ((char *) ndis_assoc_info) +
+			    ((char *)ndis_assoc_info) +
 			    ndis_assoc_info->offset_req_ies);
 	wrqu.data.length = ndis_assoc_info->resp_ie_length;
 	wireless_send_event(wnd->net_dev, IWEVASSOCRESPIE, &wrqu,
-			    ((char *) ndis_assoc_info) +
+			    ((char *)ndis_assoc_info) +
 			    ndis_assoc_info->offset_resp_ies);
 #else
 	/* we need 28 extra bytes for the format strings */
@@ -1012,26 +1003,24 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 			"association information dropped",
 			ndis_assoc_info->req_ie_length,
 			ndis_assoc_info->resp_ie_length);
-		kfree(assoc_info);
+		kfree(ndis_assoc_info);
 		TRACEEXIT2(return);
 	}
 
 	wpa_assoc_info = kmalloc(IW_CUSTOM_MAX, GFP_KERNEL);
 	if (!wpa_assoc_info) {
 		ERROR("couldn't allocate memory");
-		kfree(assoc_info);
+		kfree(ndis_assoc_info);
 		TRACEEXIT2(return);
 	}
 	p = wpa_assoc_info;
 	p += sprintf(p, "ASSOCINFO(ReqIEs=");
-	ies = ((char *)ndis_assoc_info) +
-		ndis_assoc_info->offset_req_ies;
+	ies = ((char *)ndis_assoc_info) + ndis_assoc_info->offset_req_ies;
 	for (i = 0; i < ndis_assoc_info->req_ie_length; i++)
 		p += sprintf(p, "%02x", ies[i]);
 
 	p += sprintf(p, " RespIEs=");
-	ies = ((char *)ndis_assoc_info) +
-		ndis_assoc_info->offset_resp_ies;
+	ies = ((char *)ndis_assoc_info) + ndis_assoc_info->offset_resp_ies;
 	for (i = 0; i < ndis_assoc_info->resp_ie_length; i++)
 		p += sprintf(p, "%02x", ies[i]);
 
@@ -1043,12 +1032,12 @@ static void link_status_handler(struct wrap_ndis_device *wnd)
 
 	kfree(wpa_assoc_info);
 #endif
-	kfree(assoc_info);
+	kfree(ndis_assoc_info);
 
 	get_ap_address(wnd, (char *)&wrqu.ap_addr.sa_data);
 	wrqu.ap_addr.sa_family = ARPHRD_ETHER;
 	wireless_send_event(wnd->net_dev, SIOCGIWAP, &wrqu, NULL);
-	DBGTRACE2("associatedd to " MACSTR, MAC2STR(wrqu.ap_addr.sa_data));
+	DBGTRACE2(MACSTR, MAC2STR(wrqu.ap_addr.sa_data));
 	TRACEEXIT2(return);
 }
 
@@ -1119,8 +1108,7 @@ static void wrap_ndis_worker(void *param)
 
 	DBGTRACE2("%lu", wnd->wrap_ndis_pending_work);
 
-	if (test_bit(SHUTDOWN, &wnd->wrap_ndis_pending_work) ||
-	    !test_bit(HW_INITIALIZED, &wnd->hw_status))
+	if (test_bit(SHUTDOWN, &wnd->wrap_ndis_pending_work))
 		TRACEEXIT3(return);
 
 	if (test_and_clear_bit(SET_MULTICAST_LIST, &wnd->wrap_ndis_pending_work))
@@ -1145,7 +1133,7 @@ static void wrap_ndis_worker(void *param)
 		if (res) {
 			WARNING("%s is being reset", wnd->net_dev->name);
 			res = miniport_reset(wnd);
-			DBGTRACE3("reset returns %08X", res);
+			DBGTRACE3("%08X", res);
 		}
 	}
 	TRACEEXIT3(return);
@@ -1214,13 +1202,12 @@ void get_encryption_capa(struct wrap_ndis_device *wnd)
 	ndis_key.struct_size = sizeof(ndis_key);
 	res = miniport_set_info(wnd, OID_802_11_ADD_KEY, &ndis_key,
 				ndis_key.struct_size);
-	DBGTRACE2("add key returns %08X, size = %lu",
-		  res, (unsigned long)sizeof(ndis_key));
+	DBGTRACE2("%08X, %lu", res, (unsigned long)sizeof(ndis_key));
 	if (res && res != NDIS_STATUS_INVALID_DATA)
 		TRACEEXIT1(return);
 	res = miniport_query_info(wnd, OID_802_11_ASSOCIATION_INFORMATION,
 				  &ndis_assoc_info, sizeof(ndis_assoc_info));
-	DBGTRACE1("assoc info returns %08X", res);
+	DBGTRACE1("%08X", res);
 	if (res == NDIS_STATUS_NOT_SUPPORTED)
 		TRACEEXIT1(return);
 
@@ -1242,7 +1229,6 @@ void get_encryption_capa(struct wrap_ndis_device *wnd)
 	c = (struct ndis_capability *)buf;
 	res = miniport_query_info(wnd, OID_802_11_CAPABILITY, buf, buf_len);
 	if (!(res == NDIS_STATUS_SUCCESS && c->version == 2)) {
-		DBGTRACE1("res: %X", res);
 		kfree(buf);
 		TRACEEXIT1(return);
 	}
@@ -1331,16 +1317,13 @@ wstdcall NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 				break;
 			ndis_status = miniport_set_power_state(wnd, state);
 			if (ndis_status != NDIS_STATUS_SUCCESS)
-				WARNING("setting power to %d failed: %08X",
+				WARNING("couldn't set power to %d: %08X",
 					state, ndis_status);
 			DBGTRACE2("%s: device resumed", wnd->net_dev->name);
 			irp->io_status.status = status = STATUS_SUCCESS;
 			IoCompleteRequest(irp, IO_NO_INCREMENT);
 			break;
 		} else {
-			if (test_bit(HW_SUSPENDED, &wnd->hw_status) ||
-			    test_bit(HW_HALTED, &wnd->hw_status))
-				return STATUS_FAILURE;
 			ndis_status = miniport_set_power_state(wnd, state);
 			/* TODO: handle error case */
 			if (ndis_status != NDIS_STATUS_SUCCESS)
@@ -1350,18 +1333,18 @@ wstdcall NTSTATUS NdisDispatchPower(struct device_object *fdo, struct irp *irp)
 		}
 		break;
 	case IRP_MN_QUERY_POWER:
-		if (!(wnd->attributes & NDIS_ATTRIBUTE_NO_HALT_ON_SUSPEND)) {
+		if (wnd->pm_capa) {
 			ndis_status = miniport_query_info(wnd,
 							  OID_PNP_QUERY_POWER,
 							  &state, sizeof(state));
-			DBGTRACE2("query_power to state %d returns %08X",
-				  state, ndis_status);
-			if (ndis_status == NDIS_STATUS_SUCCESS)
-				irp->io_status.status = STATUS_SUCCESS;
-			else
-				irp->io_status.status = STATUS_FAILURE;
+			DBGTRACE2("%d, %08X", state, ndis_status);
+			/* this OID must always succeed */
+			if (ndis_status != NDIS_STATUS_SUCCESS)
+				DBGTRACE1("query power returns %08X",
+					  ndis_status);
+			irp->io_status.status = STATUS_SUCCESS;
 		} else
-			irp->io_status.status = STATUS_FAILURE;
+			irp->io_status.status = STATUS_SUCCESS;
 		status = LIN2WIN2(IoPassIrpDown, wnd->nmb->pdo, irp);
 		break;
 	case IRP_MN_WAIT_WAKE:
@@ -1421,7 +1404,7 @@ wstdcall NTSTATUS NdisDispatchPnp(struct device_object *fdo, struct irp *irp)
 			status = STATUS_FAILURE;
 			break;
 		}
-		/* wnd is not valid anymore */
+		/* wnd is already freed */
 		status = LIN2WIN2(IoAsyncForwardIrp, pdo, irp);
 		IoDetachDevice(fdo);
 		IoDeleteDevice(fdo);
@@ -1554,9 +1537,9 @@ static NDIS_STATUS ndis_start_device(struct wrap_ndis_device *wnd)
 	struct wrap_device *wd;
 	struct net_device *net_dev;
 	NDIS_STATUS ndis_status;
-	mac_address mac;
 	char *buf;
 	const int buf_size = 256;
+	mac_address mac;
 
 	ndis_status = miniport_init(wnd);
 	if (ndis_status == NDIS_STATUS_NOT_RECOGNIZED)
@@ -1579,12 +1562,11 @@ static NDIS_STATUS ndis_start_device(struct wrap_ndis_device *wnd)
 	strncpy(net_dev->name, if_name, IFNAMSIZ - 1);
 	net_dev->name[IFNAMSIZ - 1] = '\0';
 
-	DBGTRACE1("%s: querying for mac", DRIVER_NAME);
 	ndis_status = miniport_query_info(wnd, OID_802_3_CURRENT_ADDRESS,
 					  mac, sizeof(mac));
 	if (ndis_status) {
 		ERROR("couldn't get mac address: %08X", ndis_status);
-		goto err_start;
+		return ndis_status;
 	}
 	DBGTRACE1("mac:" MACSTR, MAC2STR(mac));
 	memcpy(&net_dev->dev_addr, mac, ETH_ALEN);
@@ -1618,6 +1600,9 @@ static NDIS_STATUS ndis_start_device(struct wrap_ndis_device *wnd)
 		net_dev->flags |= IFF_MULTICAST;
 	else
 		net_dev->flags &= ~IFF_MULTICAST;
+#ifdef CONFIG_NET_POLL_CONTROLLER
+	net_dev->poll_controller = ndis_poll_controller;
+#endif
 
 	buf = kmalloc(buf_size, GFP_KERNEL);
 	if (!buf) {
@@ -1746,7 +1731,9 @@ static int ndis_remove_device(struct wrap_ndis_device *wnd)
 	/* In 2.4 kernels, this function is called in atomic context,
 	 * so we can't (don't need to?) wait on mutex. */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
-	down_interruptible(&wnd->tx_ring_mutex);
+	/* if device is suspended, but resume failed, tx_ring_mutex is
+	 * already locked */
+	down_trylock(&wnd->tx_ring_mutex);
 #endif
 	tx_pending = wnd->tx_ring_end - wnd->tx_ring_start;
 	if (tx_pending < 0)
@@ -1847,9 +1834,8 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	init_nmb_functions(nmb);
 	wnd->net_dev = net_dev;
 	wnd->ndis_irq = NULL;
-	init_MUTEX(&wnd->tx_ring_mutex);
-	nt_spin_lock_init(&wnd->tx_stats_lock);
-	init_MUTEX(&wnd->ndis_comm_mutex);
+	init_MUTEX_LOCKED(&wnd->tx_ring_mutex);
+	init_MUTEX_LOCKED(&wnd->ndis_comm_mutex);
 	init_waitqueue_head(&wnd->ndis_comm_wq);
 	wnd->ndis_comm_done = 0;
 	wnd->tx_ok = 0;
@@ -1875,14 +1861,12 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	wnd->infrastructure_mode = Ndis802_11Infrastructure;
 	INIT_WORK(&wnd->wrap_ndis_work, wrap_ndis_worker, wnd);
 	wnd->hw_status = 0;
-	set_bit(HW_AVAILABLE, &wnd->hw_status);
 	if (wd->driver->ndis_driver)
 		wd->driver->ndis_driver->miniport.shutdown = NULL;
 	wnd->stats_enabled = TRUE;
 	wnd->rx_csum.value = 0;
 
 	fdo->reserved = wnd;
-	nmb = wnd->nmb;
 	nmb->fdo = fdo;
 	nmb->next_device = IoAttachDeviceToDeviceStack(fdo, pdo);
 	DBGTRACE1("nmb: %p, pdo: %p, fdo: %p, attached: %p, next: %p",
@@ -1894,8 +1878,7 @@ static wstdcall NTSTATUS NdisAddDevice(struct driver_object *drv_obj,
 	drv_obj->major_func[IRP_MJ_POWER] = NdisDispatchPower;
 	drv_obj->major_func[IRP_MJ_INTERNAL_DEVICE_CONTROL] =
 		NdisDispatchDeviceControl;
-	drv_obj->major_func[IRP_MJ_DEVICE_CONTROL] =
-		NdisDispatchDeviceControl;
+	drv_obj->major_func[IRP_MJ_DEVICE_CONTROL] = NdisDispatchDeviceControl;
 	TRACEEXIT2(return STATUS_SUCCESS);
 }
 
