@@ -31,13 +31,14 @@
 #include <syslog.h>
 #include <stdlib.h>
 
+#include <linux/major.h>
 #include <linux/ioctl.h>
 
 #include "../driver/loader.h"
 
 #define PROG_NAME "loadndisdriver"
 
-#define SETTING_LEN (MAX_NDIS_SETTING_NAME_LEN+MAX_NDIS_SETTING_VALUE_LEN + 2)
+#define SETTING_LEN (MAX_SETTING_NAME_LEN + MAX_SETTING_VALUE_LEN + 2)
 
 static const char *confdir = "/etc/ndiswrapper";
 static const char *ioctl_file = "/dev/ndiswrapper";
@@ -48,38 +49,35 @@ static int debug;
 	directory only
 #endif
 
-#define error(fmt, ...) do {					\
-		syslog(LOG_KERN | LOG_ERR, "%s: %s(%d): " fmt "\n",	\
-		       PROG_NAME, __FUNCTION__, __LINE__ , ## __VA_ARGS__); \
-	} while (0)
-#define info(fmt, ...) do {						\
-		syslog(LOG_KERN | LOG_INFO, "%s: %s(%d): " fmt "\n",	\
-		       PROG_NAME, __FUNCTION__, __LINE__ , ## __VA_ARGS__); \
-	} while (0)
-
-#define dbg(fmt, ...) do { if (debug)					\
-		syslog(LOG_KERN | LOG_DEBUG, "%s: %s(%d): " fmt "\n", \
-		       PROG_NAME, __FUNCTION__, __LINE__ , ## __VA_ARGS__); \
+#define LOG_MSG(where, fmt, ...)					\
+	syslog(LOG_KERN | (where), "%s: %s(%d): " fmt "\n",		\
+	       PROG_NAME, __FUNCTION__, __LINE__ , ## __VA_ARGS__)
+#define ERROR(fmt, ...) LOG_MSG(LOG_INFO, fmt, ## __VA_ARGS__)
+#define INFO(fmt, ...) LOG_MSG(LOG_INFO, fmt, ## __VA_ARGS__)
+#define DBG(fmt, ...) do {					\
+		if (debug > 0)					\
+			LOG_MSG(LOG_INFO, fmt, ## __VA_ARGS__); \
 	} while (0)
+#define WARN(fmt, ...) LOG_MSG(LOG_INFO, fmt, ## __VA_ARGS__)
 
 /* load .sys or .bin file */
 static int load_file(char *filename, struct load_driver_file *driver_file)
 {
 	int fd;
 	size_t size;
-	void * image = NULL;
+	void *image = NULL;
 	struct stat statbuf;
 
 	char *file_basename = basename(filename);
 
 	fd = open(filename, O_RDONLY);
 	if (fd == -1) {
-		error("unable to open file: %s", strerror(errno));
+		ERROR("unable to open file: %s", strerror(errno));
 		return -EINVAL;
 	}
 
 	if (fstat(fd, &statbuf)) {
-		error("incorrect driver file '%s'", filename);
+		ERROR("incorrect driver file '%s'", filename);
 		close(fd);
 		return -EINVAL;
 	}
@@ -87,7 +85,7 @@ static int load_file(char *filename, struct load_driver_file *driver_file)
 
 	image = mmap(0, size, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (image == MAP_FAILED) {
-		error("unable to mmap driver: %s", strerror(errno));
+		ERROR("unable to mmap driver: %s", strerror(errno));
 		close(fd);
 		return -EINVAL;
 	}
@@ -116,29 +114,29 @@ static int parse_setting_line(const char *setting_line, char *setting_name,
 		return 0;
 	if ((val = strchr(s, '|')) == NULL ||
 	    (end = strchr(s, '\n')) == NULL) {
-		error("invalid setting: %s", setting_line);
+		ERROR("invalid setting: %s", setting_line);
 		return -EINVAL;
 	}
-	for (i = 0; s != val && i < MAX_NDIS_SETTING_NAME_LEN; s++, i++)
+	for (i = 0; s != val && i < MAX_SETTING_NAME_LEN; s++, i++)
 		setting_name[i] = *s;
 	setting_name[i] = 0;
 	if (*s != '|') {
-		error("invalid setting: %s", setting_line);
+		ERROR("invalid setting: %s", setting_line);
 		return -EINVAL;
 	}
 
-	for (i = 0, s++; s != end && i < MAX_NDIS_SETTING_VALUE_LEN ; s++, i++)
+	for (i = 0, s++; s != end && i < MAX_SETTING_VALUE_LEN ; s++, i++)
 		setting_val[i] = *s;
 	setting_val[i] = 0;
 	if (*s != '\n') {
-		error("invalid setting: %s", setting_line);
+		ERROR("invalid setting: %s", setting_line);
 		return -EINVAL;
 	}
-	dbg("Found setting: name=%s, val=\"%s\"", setting_name, setting_val);
+	DBG("Found setting: name=%s, val=\"%s\"", setting_name, setting_val);
 
 	// setting_val can be empty, but not name
 	if (strlen(setting_name) == 0) {
-		error("invalid setting: \"%s\"", setting_line);
+		ERROR("invalid setting: \"%s\"", setting_line);
 		return -EINVAL;
 	}
 
@@ -151,40 +149,34 @@ static int read_conf_file(char *conf_file_name, struct load_driver *driver)
 	char setting_line[SETTING_LEN];
 	struct stat statbuf;
 	FILE *config;
-	char setting_name[MAX_NDIS_SETTING_NAME_LEN];
-	char setting_value[MAX_NDIS_SETTING_VALUE_LEN];
+	char setting_name[MAX_SETTING_NAME_LEN];
+	char setting_value[MAX_SETTING_VALUE_LEN];
 	int ret, nr_settings;
-	int i, vendor, device, subvendor, subdevice, dev_bustype, conf_bustype;
+	int i, vendor, device, subvendor, subdevice, bus_type;
 
 	if (lstat(conf_file_name, &statbuf)) {
-		error("unable to open config file: %s", strerror(errno));
+		ERROR("unable to open config file: %s", strerror(errno));
 		return -EINVAL;
 	}
 
-	if (strlen(conf_file_name) == 16) {
-		i = sscanf(conf_file_name, "%04X:%04X.%d.conf",
-			   &vendor, &device, &dev_bustype);
-		if (i != 3) {
-			error("unable to parse conf file name %s (%d)",
-			      conf_file_name, i);
-			return -EINVAL;
-		}
+	if (sscanf(conf_file_name, "%04X:%04X.%X.conf",
+		   &vendor, &device, &bus_type) == 3) {
+		DBG("bus_type: %X", bus_type);
+	} else if (sscanf(conf_file_name, "%04X:%04X:%04X:%04X.%X.conf",
+			  &vendor, &device, &subvendor, &subdevice,
+			  &bus_type) == 5) {
+		DBG("bus_type: %X", bus_type);
 	} else {
-		i = sscanf(conf_file_name, "%04X:%04X:%04X:%04X.%d.conf",
-			   &vendor, &device, &subvendor, &subdevice,
-			   &dev_bustype);
-		if (i != 5) {
-			error("unable to parse conf file name %s (%d)",
-			      conf_file_name, i);
-			return -EINVAL;
-		}
+		ERROR("unable to parse conf file name %s (%d)",
+		      conf_file_name, i);
+		return -EINVAL;
 	}
 
 	nr_settings = 0;
 	driver->nr_settings = 0;
 
 	if ((config = fopen(conf_file_name, "r")) == NULL) {
-		error("unable to open config file: %s", strerror(errno));
+		ERROR("unable to open config file: %s", strerror(errno));
 		return -EINVAL;
 	}
 	while (fgets(setting_line, SETTING_LEN-1, config)) {
@@ -198,23 +190,13 @@ static int read_conf_file(char *conf_file_name, struct load_driver *driver)
 		if (ret < 0)
 			return -EINVAL;
 
-		if (strcmp(setting_name, "BusType") == 0) {
-			conf_bustype = strtol(setting_value, NULL, 10);
-			if (dev_bustype != conf_bustype) {
-				error("invalid bustype: %d(%d)",
-				      dev_bustype, conf_bustype);
-				return -EINVAL;
-			}
-		}
 		setting = &driver->settings[nr_settings];
-		strncpy(setting->name, setting_name,
-			MAX_NDIS_SETTING_NAME_LEN);
-		strncpy(setting->value, setting_value,
-			MAX_NDIS_SETTING_VALUE_LEN);
+		strncpy(setting->name, setting_name, MAX_SETTING_NAME_LEN);
+		strncpy(setting->value, setting_value, MAX_SETTING_VALUE_LEN);
 
 		nr_settings++;
-		if (nr_settings >= MAX_NDIS_SETTINGS) {
-			error("too many settings");
+		if (nr_settings >= MAX_DEVICE_SETTINGS) {
+			ERROR("too many settings");
 			return -EINVAL;
 		}
 				
@@ -222,15 +204,39 @@ static int read_conf_file(char *conf_file_name, struct load_driver *driver)
 
 	fclose(config);
 
-	if (conf_bustype == -1) {
-		error("coudn't find device type in settings");
-		return -EINVAL;
-	}			
-
 	driver->nr_settings = nr_settings;
 	return 0;
 }
 
+static int load_bin_file(int ioctl_device, char *driver_name,
+			 char *file_name)
+{
+	struct load_driver_file driver_file;
+	char lc_file_name[MAX_DRIVER_NAME_LEN];
+	int i;
+
+	DBG("loading driver %s", driver_name);
+	for (i = 0; file_name[i] && i < sizeof(lc_file_name); i++)
+		lc_file_name[i] = tolower(file_name[i]);
+	lc_file_name[i] = 0;
+	if (chdir(confdir) || chdir(driver_name)) {
+		ERROR("couldn't change to directory %s: %s",
+		      driver_name, strerror(errno));
+		return -EINVAL;
+	}
+	if (load_file(lc_file_name, &driver_file)) {
+		ERROR("couldn't open file %s", file_name);
+		return -EINVAL;
+	}
+	strncpy(driver_file.driver_name, driver_name,
+		sizeof(driver_file.driver_name));
+	if (ioctl(ioctl_device, WRAP_IOCTL_LOAD_BIN_FILE, &driver_file)) {
+		ERROR("couldn't upload bin file: %s", file_name);
+		return -EINVAL;
+	}
+	return 0;
+}
+
 /*
  * open a windows driver and pass it to the kernel module.
  * returns 0: on success, -1 on error
@@ -248,20 +254,20 @@ static int load_driver(int ioctl_device, char *driver_name,
 	nr_sys_files = 0;
 	nr_bin_files = 0;
 
-	dbg("loading driver %s", driver_name);
+	DBG("loading driver %s", driver_name);
 	if (chdir(confdir) || chdir(driver_name)) {
-		error("couldn't change to directory %s: %s",
+		ERROR("couldn't change to directory %s: %s",
 		      driver_name, strerror(errno));
 		return -EINVAL;
 	}
 	if ((driver_dir = opendir(".")) == NULL) {
-		error("couldn't open driver directory %s: %s",
+		ERROR("couldn't open driver directory %s: %s",
 		      driver_name, strerror(errno));
 		return -EINVAL;
 	}
 		
 	if ((driver = malloc(sizeof(*driver))) == NULL) {
-		error("couldn't allocate memory for driver %s", driver_name);
+		ERROR("couldn't allocate memory for driver %s", driver_name);
 		goto err;
 	}
 	memset(driver, 0, sizeof(*driver));
@@ -269,7 +275,7 @@ static int load_driver(int ioctl_device, char *driver_name,
 
 	if (read_conf_file(conf_file_name, driver) ||
 	    driver->nr_settings == 0) {
-		error("couldn't read conf file %s for driver %s",
+		ERROR("couldn't read conf file %s for driver %s",
 		      conf_file_name, driver_name);
 		goto err;
 	}
@@ -283,7 +289,7 @@ static int load_driver(int ioctl_device, char *driver_name,
 
 		if (stat(dirent->d_name, &statbuf) ||
 		    !S_ISREG(statbuf.st_mode)) {
-			error("%s in %s is not a valid file: %s",
+			ERROR("%s in %s is not a valid file: %s",
 			      dirent->d_name, driver_name, strerror(errno));
 			continue;
 		}
@@ -299,7 +305,7 @@ static int load_driver(int ioctl_device, char *driver_name,
 		if (len > 4 && strcmp(&dirent->d_name[len-4], ".sys") == 0) {
 			if (load_file(dirent->d_name,
 				      &driver->sys_files[nr_sys_files])) {
-				error("couldn't load .sys file %s",
+				ERROR("couldn't load .sys file %s",
 				      dirent->d_name);
 				goto err;
 			} else
@@ -307,30 +313,30 @@ static int load_driver(int ioctl_device, char *driver_name,
 		} else if (len > 4 &&
 			   ((strcmp(&dirent->d_name[len-4], ".bin") == 0) ||
 			     (strcmp(&dirent->d_name[len-4], ".out") == 0))) {
-			if (load_file(dirent->d_name,
-				      &driver->bin_files[nr_bin_files])) {
-				error("coudln't load .bin file %s",
-				      dirent->d_name);
-				goto err;
-			} else
-				nr_bin_files++;
+			strcpy(driver->bin_files[nr_bin_files].name,
+			       dirent->d_name);
+			strcpy(driver->bin_files[nr_bin_files].driver_name,
+			       driver_name);
+			driver->bin_files[nr_bin_files].size = 0;
+			driver->bin_files[nr_bin_files].data = NULL;
+			nr_bin_files++;
 		} else
-			error("file %s is ignored", dirent->d_name);
+			ERROR("file %s is ignored", dirent->d_name);
 
-		if (nr_sys_files == MAX_PE_IMAGES) {
-			error("too many .sys files for driver %s",
+		if (nr_sys_files == MAX_DRIVER_PE_IMAGES) {
+			ERROR("too many .sys files for driver %s",
 			      driver_name);
 			goto err;
 		}
-		if (nr_bin_files == MAX_NDIS_BIN_FILES) {
-			error("too many .bin files for driver %s",
+		if (nr_bin_files == MAX_DRIVER_BIN_FILES) {
+			ERROR("too many .bin files for driver %s",
 			      driver_name);
 			goto err;
 		}
 	}
 
 	if (nr_sys_files == 0) {
-		error("coudln't find valid drivers files for driver %s",
+		ERROR("coudln't find valid drivers files for driver %s",
 		      driver_name);
 		goto err;
 	}
@@ -338,12 +344,10 @@ static int load_driver(int ioctl_device, char *driver_name,
 	driver->nr_bin_files = nr_bin_files;
 	strncpy(driver->conf_file_name, conf_file_name,
 		sizeof(driver->conf_file_name));
-
-	if (ioctl(ioctl_device, NDIS_LOAD_DRIVER, driver))
+	if (ioctl(ioctl_device, WRAP_IOCTL_LOAD_DRIVER, driver))
 		goto err;
-
 	closedir(driver_dir);
-	dbg("driver %s loaded", driver_name);
+	DBG("driver %s loaded", driver_name);
 	free(driver);
 	return 0;
 
@@ -351,10 +355,10 @@ err:
 	if (driver_dir)
 		closedir(driver_dir);
 	for (i = 0; i < nr_sys_files; i++)
-		free(driver->sys_files[i].data);
+		munmap(driver->sys_files[i].data, driver->sys_files[i].size);
 	for (i = 0; i < nr_bin_files; i++)
-		free(driver->bin_files[i].data);
-	error("couldn't load driver %s", driver_name);
+		munmap(driver->bin_files[i].data, driver->bin_files[i].size);
+	ERROR("couldn't load driver %s", driver_name);
 	free(driver);
 	return -1;
 }
@@ -376,35 +380,40 @@ static int duplicate_device(struct load_device *device, int n,
 }
 
 /* add all devices (based on conf files) for a given driver */
-static int add_driver_devices(DIR *dir, char *driver_name,
-			      int from, struct load_device devices[])
+static int add_driver_devices(DIR *dir, char *driver_name, int from,
+			      struct load_device devices[])
 {
 	struct dirent *dirent;
 	int n;
 
 	n = from;
 	if (!dir || !driver_name) {
-		error("invalid driver");
+		ERROR("invalid driver");
 		return n;
 	}
 
-	dbg("adding devices for driver %s", driver_name);
+	DBG("adding devices for driver %s", driver_name);
 	while ((dirent = readdir(dir))) {
 		int len;
 
+		if (n >= MAX_WRAP_DEVICES) {
+			ERROR("too many devices; increase MAX_WRAP_DEVICES "
+			      "in ndiswrapper.h and recompile or remove "
+			      "unnecessary .conf files under '%s'", confdir);
+			break;
+		}
 		if (strcmp(dirent->d_name, ".") == 0 ||
 		    strcmp(dirent->d_name, "..") == 0)
 			continue;
 
 		len = strlen(dirent->d_name);
-		if (len > 5 &&
-			   strcmp(&dirent->d_name[len-5], ".conf") == 0) {
+		if (len > 5 && strcmp(&dirent->d_name[len-5], ".conf") == 0) {
 			struct stat statbuf;
 			char *s;
 			struct load_device *device;
 
 			if (lstat(dirent->d_name, &statbuf)) {
-				error("unable to open config file: %s",
+				ERROR("unable to open config file: %s",
 				      strerror(errno));
 				continue;
 			}
@@ -414,31 +423,26 @@ static int add_driver_devices(DIR *dir, char *driver_name,
 			s[strlen(s)-5] = 0;
 
 			device = &devices[n];
-			if (strlen(s) == 11 &&
-			    sscanf(s, "%04x:%04x.%1d", &device->vendor,
-				   &device->device, &device->bustype) == 3) {
+			if (strlen(s) >= 11 &&
+			    sscanf(s, "%04x:%04x.%X", &device->vendor,
+				   &device->device, &device->bus_type) == 3) {
+				DBG("bus_type: %X", device->bus_type);
 				device->subvendor = DEV_ANY_ID;
 				device->subdevice = DEV_ANY_ID;
-			} else if (strlen(s) == 21 &&
-				   sscanf(s, "%04x:%04x:%04x:%04x.%1d",
+			} else if (strlen(s) >= 21 &&
+				   sscanf(s, "%04x:%04x:%04x:%04x.%X",
 					  &device->vendor, &device->device,
 					  &device->subvendor,
 					  &device->subdevice,
-					  &device->bustype) == 5) {
-				;
+					  &device->bus_type) == 5) {
+				DBG("bus_type: %X", device->bus_type);
 			} else {
-				error("file %s is not valid - ignored",
+				ERROR("file %s is not valid - ignored",
 				      dirent->d_name);
 				continue;
 			}
-			if (device->bustype != NDIS_PCI_BUS &&
-			    device->bustype != NDIS_USB_BUS) {
-				error("incorrect bus type %d",
-				      device->bustype);
-				continue;
-			}
 			if (duplicate_device(device, n, devices))
-				dbg("device %04X:%04X is duplicate - ignored",
+				DBG("device %04X:%04X is duplicate - ignored",
 				    device->vendor, device->device);
 			else {
 				strncpy(device->driver_name, driver_name,
@@ -448,14 +452,14 @@ static int add_driver_devices(DIR *dir, char *driver_name,
 				strncat(device->conf_file_name, ".conf",
 					sizeof(device->conf_file_name) -
 					strlen(device->conf_file_name));
-				dbg("device %04X:%04X:%04X:%04X is added",
+				DBG("device %04X:%04X:%04X:%04X is added",
 				    device->vendor, device->device,
 				    device->subvendor, device->subdevice);
 				n++;
 			}
 		}
 	}
-	dbg("total number of devices added: %d", n);
+	DBG("number of devices in %s: %d", driver_name, n - from);
 	return n;
 }
 
@@ -473,23 +477,23 @@ static int load_all_devices(int ioctl_device)
 	struct load_devices load_devices;
 
 	if (chdir(confdir)) {
-		error("directory %s is not valid: %s",
+		ERROR("directory %s is not valid: %s",
 		      confdir, strerror(errno));
 		return -EINVAL;
 	}
 	if ((dir = opendir(confdir)) == NULL) {
-		error("directory %s is not valid: %s",
+		ERROR("directory %s is not valid: %s",
 		      confdir, strerror(errno));
 		return -EINVAL;
 	}
 
-	devices = malloc(sizeof(*devices) * MAX_DEVICES);
+	devices = malloc(sizeof(*devices) * MAX_WRAP_DEVICES);
 	if (!devices) {
-		error("couldn't allocate memory");
+		ERROR("couldn't allocate memory");
 		return -EINVAL;
 	}
 	loaded = 0;
-	while((dirent = readdir(dir))) {
+	while ((dirent = readdir(dir))) {
 		if (strcmp(dirent->d_name, ".") == 0 ||
 		    strcmp(dirent->d_name, "..") == 0)
 			continue;
@@ -497,27 +501,26 @@ static int load_all_devices(int ioctl_device)
 		if (stat(dirent->d_name, &statbuf) ||
 		    (!S_ISDIR(statbuf.st_mode)))
 			continue;
-
 		if ((driver = opendir(dirent->d_name)) == NULL) {
-			error("directory %s is not valid: %s",
+			ERROR("directory %s is not valid: %s",
 			      dirent->d_name, strerror(errno));
 			continue;
 		}
 		if (chdir(dirent->d_name)) {
-			error("directory %s is not valid: %s",
+			ERROR("directory %s is not valid: %s",
 			      dirent->d_name, strerror(errno));
 			closedir(driver);
 			continue;
 		}
-		loaded = add_driver_devices(driver, dirent->d_name,
-					    loaded, devices);
+		loaded = add_driver_devices(driver, dirent->d_name, loaded,
+					    devices);
 		chdir("..");
 		closedir(driver);
 	}
 	closedir(dir);
 
 	if (loaded == 0) {
-		error("no valid NDIS drives found in %s; you may need to"
+		ERROR("no valid drives found in %s; you may need to"
 		      " reinstall Windows drivers", confdir);
 		free(devices);
 		return -1;
@@ -525,19 +528,20 @@ static int load_all_devices(int ioctl_device)
 	load_devices.count = loaded;
 	load_devices.devices = devices;
 
-	res = ioctl(ioctl_device, NDIS_REGISTER_DEVICES, &load_devices);
+	res = ioctl(ioctl_device, WRAP_IOCTL_REGISTER_DEVICES, &load_devices);
+	DBG("res: %d", res);
 	free(devices);
 
 	if (res) {
-		error("couldn't load devices");
+		ERROR("couldn't load devices");
 		return -1;
 	}
-
+	DBG("number of devices loaded: %d", loaded);
 	return 0;
 }
 
 /*
-  * we need a device to use ioctl to communicate with ndiswrapper module
+  * we need a device to use ioctl to communicate with wrapper module
   * we create a device in /dev instead of /tmp as some distributions don't
   * allow creation of devices in /tmp
   */
@@ -547,14 +551,14 @@ static int get_ioctl_device()
 	char line[64];
 	FILE *proc_misc;
 
-	/* get minor device number used by ndiswrapper driver */
+	/* get minor device number used by wrapper driver */
 	proc_misc = fopen("/proc/misc", "r");
 	if (!proc_misc)
 		return -1;
 	minor_dev = -1;
 	while (fgets(line, sizeof(line), proc_misc)) {
 		if (strstr(line, DRIVER_NAME)) {
-			long i = strtol(line, 0, 10);
+			long i = strtol(line, NULL, 10);
 			if (i != LONG_MAX && i != LONG_MIN) {
 				minor_dev = i;
 				break;
@@ -564,14 +568,15 @@ static int get_ioctl_device()
 	fclose(proc_misc);
 
 	if (minor_dev == -1) {
-		error("couldn't find ndiswrapper in /proc/misc; "
-		      "is ndiswrapper module loaded?");
+		ERROR("couldn't find wrapper in /proc/misc; "
+		      "is module loaded?");
 		return -1;
 	}
 
 	unlink(ioctl_file);
-	if (mknod(ioctl_file, S_IFCHR | 0600, 10 << 8 | minor_dev) == -1) {
-		error("couldn't create file %s: %s",
+	if (mknod(ioctl_file, S_IFCHR | 0600, MISC_MAJOR << 8 | minor_dev) ==
+	    -1) {
+		ERROR("couldn't create file %s: %s",
 		      ioctl_file, strerror(errno));
 		return -1;
 	}
@@ -580,7 +585,7 @@ static int get_ioctl_device()
 	unlink(ioctl_file);
 
 	if (fd == -1) {
-		error("couldn't open file %s: %s",
+		ERROR("couldn't open file %s: %s",
 		      ioctl_file, strerror(errno));
 		return -1;
 	}
@@ -595,18 +600,27 @@ static int get_ioctl_device()
 int main(int argc, char *argv[0])
 {
 	int i, ioctl_device, res;
+	char *cmd;
 
 	openlog(PROG_NAME, LOG_PERROR | LOG_CONS, LOG_KERN | LOG_DEBUG);
 
+	DBG("argc: %d", argc);
+
+	if (argc == 2 && (strncmp(argv[1], "-v", 2) == 0 ||
+			  strncmp(argv[1], "--v", 3) == 0)) {
+		printf("version: %s\n", UTILS_VERSION);
+		return 0;
+	}
 	if (argc < 4) {
 		res = 1;
 		goto out;
 	}
 
+	cmd = argv[1];
 	i = -1;
-	i = atoi(argv[1]);
+	i = atoi(argv[2]);
 	if (i < 0) {
-		error("invalid debug value %d", i);
+		ERROR("invalid debug value %d", i);
 		res = 2;
 		goto out;
 	} else
@@ -614,36 +628,44 @@ int main(int argc, char *argv[0])
 
 	ioctl_device = get_ioctl_device();
 	if (ioctl_device == -1) {
-		error("unable to open ioctl device %s", ioctl_file);
+		ERROR("unable to open ioctl device %s", ioctl_file);
 		res = 5;
 		goto out;
 	}
 
-	if (atof(argv[2]) != atof(UTILS_VERSION)) {
-		error("version %s doesn't match driver version %s",
-		      UTILS_VERSION, argv[2]);
+	if (atof(argv[3]) != atof(UTILS_VERSION)) {
+		ERROR("version %s doesn't match driver version %s",
+		      UTILS_VERSION, argv[3]);
 		res = 6;
 		goto out;
 	}
 
-	if (strcmp(argv[3], "-a") == 0) {
+	if (strcmp(cmd, WRAP_CMD_LOAD_DEVICES) == 0) {
 		if (load_all_devices(ioctl_device))
 			res = 7;
 		else
 			res = 0;
-	} else {
+	} else if (strcmp(cmd, WRAP_CMD_LOAD_DRIVER) == 0) {
 		/* load specific driver and conf file */
-		if (argc != 5) {
-			error("incorrect usage of %s (%d)", argv[0], argc);
+		if (argc != 6) {
+			ERROR("incorrect usage of %s (%d)", argv[0], argc);
 			res = 11;
 			goto out;
 		}
-		res = load_driver(ioctl_device, argv[3], argv[4]);
+		res = load_driver(ioctl_device, argv[4], argv[5]);
+	} else if (strcmp(cmd, WRAP_CMD_LOAD_BIN_FILE) == 0) {
+		/* load specific driver and conf file */
+		if (argc != 6) {
+			ERROR("incorrect usage of %s (%d)", argv[0], argc);
+			res = 12;
+			goto out;
+		}
+		res = load_bin_file(ioctl_device, argv[4], argv[5]);
 	}
-
 out:
 	if (ioctl_device != -1)
 		close(ioctl_device);
 	closelog();
 	return res;
 }
+
